import 'package:flutter/services.dart';

import '../pano_rtc.dart';
import 'enum_converter.dart';

class RtcAnnotation with RtcAnnotationInterface {
  final String _channelId;
  EventChannel _eventChannel;
  MethodChannel _methodChannel;
  AnnotationEventHandler _handler;

  RtcAnnotation(this._channelId) {
    _methodChannel = MethodChannel("pano_rtc/api_annotation_$_channelId");
    _eventChannel = EventChannel("pano_rtc/events_annotation_$_channelId");

    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  void setEventHandler(AnnotationEventHandler handler) {
    _handler = handler;
  }

  Future<T> _invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  @override
  Future<ResultCode> clearContents() {
    return _invokeMethod('clearContents');
  }

  @override
  Future<ResultCode> clearUserContents(int userId) {
    return _invokeMethod('clearUserContents', {'userId': userId});
  }

  @override
  Future<ResultCode> redo() {
    return _invokeMethod('redo');
  }

  @override
  Future<ResultCode> setColor(WBColor color) {
    return _invokeMethod('setColor', {'color': color.toJson()});
  }

  @override
  Future<ResultCode> setFontSize(int size) {
    return _invokeMethod('setFontSize', {'size': size});
  }

  @override
  Future<ResultCode> setFontStyle(WBFontStyle style) {
    return _invokeMethod(
        'setFontStyle', {'style': WBFontStyleConverter(style).value()});
  }

  @override
  Future<ResultCode> setLineWidth(int size) {
    return _invokeMethod('setLineWidth', {'size': size});
  }

  @override
  Future<ResultCode> setRoleType(WBRoleType type) {
    return _invokeMethod(
        'setRoleType', {'type': WBRoleTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> setToolType(WBToolType type) {
    return _invokeMethod(
        'setToolType', {'type': WBToolTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> setVisible(bool visible) {
    return _invokeMethod('setVisible', {'visible': visible});
  }

  @override
  Future<ResultCode> snapshot(String outputDir) {
    return _invokeMethod('snapshot', {'outputDir': outputDir});
  }

  @override
  Future<ResultCode> startAnnotation(RtcSurfaceViewModel viewModel) {
    return viewModel.invokeMethod('startAnnotation', {'channelId': _channelId});
  }

  @override
  Future<ResultCode> stopAnnotation() {
    return _invokeMethod('stopAnnotation');
  }

  @override
  Future<ResultCode> undo() {
    return _invokeMethod('undo');
  }
}

mixin RtcAnnotationInterface {
  ///@~english
  ///@brief Set annotation role type.
  ///@param type  The annotation role type.
  ///@~chinese
  ///@brief 设置标注角色类型
  ///@param type 标注角色
  Future<ResultCode> setRoleType(WBRoleType type);

  ///@~english
  ///@brief Start annotation  and set render window
  ///@param window  Platform specified window object, it should HWND on windows platform.
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@note Please make sure that annotation service is available before start annotation
  ///@~chinese
  ///@brief 开启标注并且设置显示窗口
  ///@param viewModel  平台相关的窗口对象， 在windows平台， 该参数必须是HWND对象。
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  ///@note 开启标注前需要保证白板服务是正常的。
  Future<ResultCode> startAnnotation(RtcSurfaceViewModel viewModel);

  ///@~english
  ///@brief Stop the annotation.
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 停止标注
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> stopAnnotation();

  ///@~english
  ///@brief Set annotation view visible/invisible.
  ///@param visible visible or not
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@note annotation view is visible default
  ///@~chinese
  ///@brief 设置标注视图是否可见
  ///@param visible 是否可见
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  ///@note 标注视图默认是可见的
  Future<ResultCode> setVisible(bool visible);

  ///@~english
  ///@brief set tool type
  ///@param type tool type
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 设置工具类型
  ///@paran type 工具类型
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> setToolType(WBToolType type);

  ///@~english
  ///@brief set line width
  ///@param size line width. Valid value range [1, 20]
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 设置线宽
  ///@paran size 线宽。 有效值范围 [1, 20]
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> setLineWidth(int size);

  ///@~english
  ///@brief Set color
  ///@param color  Color
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 设置颜色
  ///@paran color 颜色
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> setColor(WBColor color);

  ///@~english
  ///@brief Set font style
  ///@param style font style
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 设置字体样式
  ///@paran style 字体样式
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> setFontStyle(WBFontStyle style);

  ///@~english
  ///@brief Set font size
  ///@param size font size
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 设置字体大小
  ///@paran size 字体大小
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> setFontSize(int size);

  ///@~english
  ///@brief undo
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 撤消上一次操作
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> undo();

  ///@~english
  ///@brief redo
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@~chinese
  ///@brief 重做标注的上一次被撤销操作
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  Future<ResultCode> redo();

  ///@~english
  ///@brief clear annotation content by specific user ID
  ///@param userId user ID
  ///@return
  ///   - OK: Success
  ///   - NO_PRIVILEGE: need ADMIN role to call this API
  ///   - Others: Fail
  ///@note ADMIN role is required if the userId is not local user
  ///@~chinese
  ///@brief 清除指定用户标注内容
  ///@param userId 用户ID
  ///@return
  ///  - OK： 成功
  ///  - NO_PRIVILEGE: 没有权限
  ///  - Others: 失败
  ///@note 只有 ADMIN 角色才可以清除非本地用户的内容
  Future<ResultCode> clearUserContents(int userId);

  ///@~english
  ///@brief clear annotation content
  ///@return
  ///   - OK: Success
  ///   - NO_PRIVILEGE: need ADMIN role to call this API
  ///   - Others: Fail
  ///@note this API need ADMIN role
  ///@~chinese
  ///@brief 清除标注内容，需要 ADMIN 角色才可调用成功
  ///@return
  ///  - OK： 成功
  ///  - NO_PRIVILEGE: 没有权限
  ///  - Others: 失败
  ///@note 此接口只有 ADMIN 角色才可调用
  Future<ResultCode> clearContents();

  ///@~english
  ///@brief save annotation contents to image.
  ///@param outputDir output directory
  ///@return
  ///   - OK: Success
  ///   - Others: Fail
  ///@note snapshot result and image filename is returned in callback onSnapshotComplete
  ///@~chinese
  ///@brief 保存标注内容到图像。
  ///@param outputDir 输出路径
  ///@return
  ///  - OK： 成功
  ///  - Others: 失败
  ///@note 快照结果和图像文件名通过回调函数onSnapshotComplete返回
  Future<ResultCode> snapshot(String outputDir);
}
