import 'package:flutter/services.dart';
import 'package:pano_rtc/src/rtc_annotation.dart';

import '../pano_rtc.dart';
import 'enum_converter.dart';

class RtcAnnotationManager with RtcAnnotationManagerInterface {
  final MethodChannel _methodChannel =
      MethodChannel('pano_rtc/api_annotationMgr');
  final EventChannel _eventChannel =
      EventChannel('pano_rtc/events_annotationMgr');
  AnnotationMgrEventHandler _handler;
  Map<String, RtcAnnotation> _videoAnnoMap = Map();
  Map<String, RtcAnnotation> _shareAnnoMap = Map();

  RtcAnnotationManager() {
    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  Future<T> _invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  void setEventHandler(AnnotationMgrEventHandler handler) {
    _handler = handler;
  }

  @override
  Future<RtcAnnotation> getShareAnnotation(int userId) async {
    String key = userId.toString();
    RtcAnnotation rtcAnnotation = _shareAnnoMap[key];
    if (rtcAnnotation == null) {
      rtcAnnotation = RtcAnnotation(key);
      _shareAnnoMap[key] = rtcAnnotation;
      _invokeMethod('getShareAnnotation', {'userId': userId});
    }
    return rtcAnnotation;
  }

  @override
  Future<RtcAnnotation> getVideoAnnotation(int userId, int streamId) async {
    String key = '$userId-$streamId';
    RtcAnnotation rtcAnnotation = _videoAnnoMap[key];
    if (rtcAnnotation == null) {
      rtcAnnotation = RtcAnnotation(key);
      _videoAnnoMap[key] = rtcAnnotation;
      _invokeMethod('getVideoAnnotation', {'userId': userId, 'streamId': streamId});
    }
    return rtcAnnotation;
  }
}

mixin RtcAnnotationManagerInterface {
  ///@~english
  ///@brief get video annotation object.
  ///@param userId  User ID
  ///@param streamId Stream ID
  ///@return
  ///  - non-null: a pointer to the video annotation object
  ///  - others: Failure
  ///@note DO NOT delete this pointer,  AnnotationManager can handle its life cycle.
  ///@~chinese
  ///@brief 获取视频标注对象
  ///@param userId 用户ID
  ///@param streamId 视频流ID
  ///@return
  /// - 非空指针： 指向视频标注对象的指针。
  /// - 空指针: 失败
  ///@note 不要删除此指针， AnnotationManager会维护其生命周期。
  Future<RtcAnnotation> getVideoAnnotation(int userId, int streamId);

  ///@~english
  ///@brief get share annotation object.
  ///@param userId  User ID
  ///@return
  ///  - non-null: a pointer to the share annotation object
  ///  - others: Failure
  ///@note DO NOT delete this pointer,  AnnotationManager can handle its life cycle.
  ///@~chinese
  ///@brief 获取共享标注对象
  ///@param userId 用户ID
  ///@return
  /// - 非空指针： 指向共享标注对象的指针。
  /// - 空指针: 失败
  ///@note 不要删除此指针， AnnotationManager会维护其生命周期。
  Future<RtcAnnotation> getShareAnnotation(int userId);
}
