import 'dart:async';
import 'dart:math';

import 'package:flutter/services.dart';
import 'package:pano_rtc/src/rtc_whiteboard.dart';

import 'enum_converter.dart';
import 'rtc_annotationManager.dart';
import 'rtc_enums.dart';
import 'rtc_events.dart';
import 'rtc_networkManager.dart';
import 'rtc_objects.dart';
import 'rtc_videoStreamManager.dart';
import 'rtc_view.dart';

/// RtcEngine is the main class of the PanoRtc SDK.
class RtcEngineKit with RtcEngineKitInterface {
  static const MethodChannel _methodChannel =
      MethodChannel('pano_rtc/api_engine');
  static const EventChannel _eventChannel =
      EventChannel('pano_rtc/events_engine');

  static RtcEngineKit _engine;

  RtcEngineEventHandler _handler;
  Map<String, RtcWhiteboard> _whiteboardMap = Map();
  RtcVideoStreamManager _videoStreamManager;
  RtcAnnotationManager _annotationManager;
  RtcNetworkManager _networkManager;

  RtcEngineKit._() {
    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  Future<T> _invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  /// @~english
  /// @brief Creating an RtcEngineKit object.
  /// @param config    RtcEngineConfig object.
  /// @return RtcEngineKit object.
  /// @note If the object creation fails, an null object is returned.
  /// @~chinese
  /// @brief 创建一个 RtcEngineKit 对象。
  /// @param config    RtcEngineConfig 对象.
  /// @return RtcEngineKit 对象
  /// @note 如果对象创建失败，将返回空对象。
  static Future<RtcEngineKit> engine(RtcEngineConfig config) async {
    if (_engine != null) return _engine;
    await _methodChannel.invokeMethod('create', {'config': config.toJson()});
    _engine = RtcEngineKit._();
    return _engine;
  }

  @override
  Future<void> destroy() {
    _engine = null;
    _whiteboardMap.clear();
    _videoStreamManager = null;
    _annotationManager = null;
    _networkManager = null;
    return _invokeMethod('destroy');
  }

  /// Sets the engine event handler.
  ///
  /// After setting the engine event handler, you can listen for engine events and receive the statistics of the corresponding [RtcEngineKit] instance.
  ///
  /// **Parameter** [handler] The event handler.
  void setEventHandler(RtcEngineEventHandler handler) {
    _handler = handler;
  }

  /// @~english
  /// @brief Set log output level.
  /// @param level   PanoLogLevel enum type.
  /// @~chinese
  /// @brief 设置日志输出级别。
  /// @param level   PanoLogLevel 枚举类型.
  static Future<void> setLogLevel(LogLevel level) {
    return _methodChannel.invokeMethod(
        'setLogLevel', {'level', LogLevelConverter(level).value()});
  }

  /// @~english
  /// @brief Retrieves the PANO SDK version.
  /// @return The PANO SDK version, format as 1.0.1.
  /// @~chinese
  /// @brief 返回 PANO SDK 的版本信息
  /// @return PANO SDK 版本字符串，比如 1.0.1。
  static Future<String> getSdkVersion() {
    return _methodChannel.invokeMethod('getSdkVersion');
  }

  @override
  Future<ResultCode> updateConfig(RtcEngineConfig config) {
    return _invokeMethod('updateConfig', {'config': config.toJson()});
  }

  @override
  Future<ResultCode> joinChannel(String token, String channelId, int userId,
      {RtcChannelConfig config}) {
    if (config == null) {
      config = RtcChannelConfig();
    }
    return _invokeMethod('joinChannel', {
      'token': token,
      'channelId': channelId,
      'userId': userId,
      'config': config.toJson()
    });
  }

  @override
  Future<void> leaveChannel() {
    return _invokeMethod('leaveChannel');
  }

  @override
  Future<ResultCode> startAudio() {
    return _invokeMethod('startAudio');
  }

  @override
  Future<ResultCode> stopAudio() {
    return _invokeMethod('stopAudio');
  }

  @override
  Future<ResultCode> startVideo(RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config}) {
    if (config == null) {
      config = RtcRenderConfig();
    }
    return viewModel.invokeMethod('startVideo', {'config': config.toJson()});
  }

  @override
  Future<ResultCode> subscribeAudio(int userId) {
    return _invokeMethod('subscribeAudio', {'userId': userId});
  }

  @override
  Future<ResultCode> unsubscribeAudio(int userId) {
    return _invokeMethod('unsubscribeAudio', {'userId': userId});
  }

  @override
  Future<ResultCode> subscribeVideo(int userId, RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config}) {
    if (config == null) {
      config = RtcRenderConfig();
    }
    return viewModel.invokeMethod(
        'subscribeVideo', {'userId': userId, 'config': config.toJson()});
  }

  @override
  Future<ResultCode> unsubscribeVideo(int userId) {
    return _invokeMethod('unsubscribeVideo', {'userId': userId});
  }

  @override
  Future<ResultCode> subscribeScreen(
      int userId, RtcSurfaceViewModel viewModel) {
    return viewModel.invokeMethod('subscribeScreen', {'userId': userId});
  }

  @override
  Future<ResultCode> unsubscribeScreen(int userId) {
    return _invokeMethod('unsubscribeScreen', {'userId': userId});
  }

  @override
  Future<ResultCode> updateScreenScaling(int userId, ScreenScalingRatio ratio) {
    return _invokeMethod('updateScreenScaling', {
      'userId': userId,
      'ratio': ScreenScalingRatioConverter(ratio).value()
    });
  }

  @override
  Future<ResultCode> updateScreenScalingWithFocus(
      int userId, double ratio, Point<int> focus) {
    return _invokeMethod('updateScreenScalingWithFocus', {
      'userId': userId,
      'ratio': ratio,
      'focus': {'x': focus.x, 'y': focus.y}
    });
  }

  @override
  Future<ResultCode> updateScreenMoving(int userId, Point<int> distance) {
    return _invokeMethod('updateScreenMoving', {
      'userId': userId,
      'distance': {'x': distance.x, 'y': distance.y}
    });
  }

  @override
  Future<ResultCode> muteAudio() {
    return _invokeMethod('muteAudio');
  }

  @override
  Future<ResultCode> unmuteAudio() {
    return _invokeMethod('unmuteAudio');
  }

  @override
  Future<ResultCode> muteVideo() {
    return _invokeMethod('muteVideo');
  }

  @override
  Future<ResultCode> unmuteVideo() {
    return _invokeMethod('unmuteVideo');
  }

  @override
  Future<ResultCode> setMicrophoneMuteStatus(bool enable) {
    return _invokeMethod('setMicrophoneMuteStatus', {'enable': enable});
  }

  @override
  Future<ResultCode> setAudioDeviceVolume(int volume, AudioDeviceType type) {
    return _invokeMethod('setAudioDeviceVolume',
        {'volume': volume, 'type': AudioDeviceTypeConverter(type).value()});
  }

  @override
  Future<int> getAudioDeviceVolume(AudioDeviceType type) {
    return _invokeMethod('getAudioDeviceVolume',
        {'type': AudioDeviceTypeConverter(type).value()});
  }

  @override
  Future<int> getRecordingLevel() {
    return _invokeMethod('getRecordingLevel');
  }

  @override
  Future<int> getPlayoutLevel() {
    return _invokeMethod('getPlayoutLevel');
  }

  @override
  Future<ResultCode> setLoudspeakerStatus(bool enable) {
    return _invokeMethod('setLoudspeakerStatus', {'enable': enable});
  }

  @override
  Future<bool> isEnabledLoudspeaker() {
    return _invokeMethod('isEnabledLoudspeaker');
  }

  @override
  Future<ResultCode> switchCamera() {
    return _invokeMethod('switchCamera');
  }

  @override
  Future<bool> isFrontCamera() {
    return _invokeMethod('isFrontCamera');
  }

  @override
  Future<String> getCameraDeviceId(bool frontCamera) {
    return _invokeMethod('getCameraDeviceId', {'frontCamera': frontCamera});
  }

  @override
  Future<ResultCode> startPreview(RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config}) {
    if (config == null) {
      config = RtcRenderConfig();
    }
    return viewModel.invokeMethod('startPreview', {'config': config.toJson()});
  }

  @override
  Future<ResultCode> stopPreview() {
    return _invokeMethod('stopPreview');
  }

  @override
  Future<ResultCode> createAudioMixingTask(int taskId, String filename) {
    return _invokeMethod(
        'createAudioMixingTask', {'taskId': taskId, 'filename': filename});
  }

  @override
  Future<ResultCode> destroyAudioMixingTask(int taskId) {
    return _invokeMethod('destroyAudioMixingTask', {'taskId': taskId});
  }

  @override
  Future<ResultCode> startAudioMixingTask(
      int taskId, RtcAudioMixingConfig config) {
    return _invokeMethod(
        'startAudioMixingTask', {'taskId': taskId, 'config': config.toJson()});
  }

  @override
  Future<ResultCode> updateAudioMixingTask(
      int taskId, RtcAudioMixingConfig config) {
    return _invokeMethod(
        'updateAudioMixingTask', {'taskId': taskId, 'config': config.toJson()});
  }

  @override
  Future<ResultCode> stopAudioMixingTask(int taskId) {
    return _invokeMethod('stopAudioMixingTask', {'taskId': taskId});
  }

  @override
  Future<ResultCode> resumeAudioMixing(int taskId) {
    return _invokeMethod('resumeAudioMixing', {'taskId': taskId});
  }

  @override
  Future<ResultCode> pauseAudioMixing(int taskId) {
    return _invokeMethod('pauseAudioMixing', {'taskId': taskId});
  }

  @override
  Future<int> getAudioMixingDuration(int taskId) {
    return _invokeMethod('getAudioMixingDuration', {'taskId': taskId});
  }

  @override
  Future<int> getAudioMixingCurrentTimestamp(int taskId) {
    return _invokeMethod('getAudioMixingCurrentTimestamp', {'taskId': taskId});
  }

  @override
  Future<ResultCode> seekAudioMixing(int taskId, int timestampMs) {
    return _invokeMethod(
        'seekAudioMixing', {'taskId': taskId, 'timestampMs': timestampMs});
  }

  @override
  Future<ResultCode> snapshotVideo(String outputDir, int userId,
      {RtcSnapshotVideoOption option}) {
    if (option == null) {
      option = RtcSnapshotVideoOption();
    }
    return _invokeMethod('snapshotVideo',
        {'outputDir': outputDir, 'userId': userId, 'option': option.toJson()});
  }

  @override
  Future<RtcWhiteboard> whiteboardEngine() async {
    var channelId = await _invokeMethod('whiteboardEngine');
    var cachedWhiteboard = _whiteboardMap[channelId];
    if (cachedWhiteboard == null) {
      cachedWhiteboard = RtcWhiteboard(channelId);
      _whiteboardMap[channelId] = cachedWhiteboard;
    }
    return cachedWhiteboard;
  }

  @override
  Future<ResultCode> switchWhiteboardEngine(String whiteboardId) {
    return _invokeMethod(
        'switchWhiteboardEngine', {'whiteboardId': whiteboardId});
  }

  @override
  Future<ResultCode> startAudioDumpWithFilePath(
      String filePath, int maxFileSize) {
    return _invokeMethod('startAudioDumpWithFilePath',
        {'filePath': filePath, 'maxFileSize': maxFileSize});
  }

  @override
  Future<ResultCode> stopAudioDump() {
    return _invokeMethod('stopAudioDump');
  }

  @override
  Future<ResultCode> sendFeedback(FeedbackInfo info) {
    return _invokeMethod('sendFeedback', {'info': info.toJson()});
  }

  @override
  Future<ResultCode> setOption(option, OptionType type) {
    var params = Map<String, dynamic>();
    params["type"] = OptionTypeConverter(type).value();
    var isValid = true;
    switch (type) {
      case OptionType.FaceBeautify:
        if (option is FaceBeautifyOption) {
          params["option"] = option.toJson();
        } else {
          isValid = false;
        }
        break;
      case OptionType.UploadLogs:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case OptionType.UploadAudioDump:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case OptionType.AudioEqualizationMode:
        if (option is AudioEqualizationMode) {
          params["option"] = AudioEqualizationModeConverter(option).value();
        } else {
          isValid = false;
        }
        break;
      case OptionType.AudioReverbMode:
        if (option is AudioReverbMode) {
          params["option"] = AudioReverbModeConverter(option).value();
        } else {
          isValid = false;
        }
        break;
      case OptionType.VideoFrameRate:
        if (option is VideoFrameRateType) {
          params["option"] = VideoFrameRateTypeConverter(option).value();
        } else {
          isValid = false;
        }
        break;
      case OptionType.AudioEarMonitoring:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case OptionType.BuiltinTransform:
        isValid = false;
        break;
      case OptionType.UploadLogsAtFailure:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case OptionType.CpuAdaption:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case OptionType.AudioProfile:
        if (option is RtcAudioProfile) {
          params["option"] = option.toJson();
        } else {
          isValid = false;
        }
        break;
      case OptionType.QuadTransform:
        if (option is QuadTransformOption) {
          params["option"] = option.toJson();
        } else {
          isValid = false;
        }
        break;
      default:
        isValid = false;
    }

    if (!isValid) return Future.value(ResultCode.InvalidArgs);

    return _invokeMethod('setOption', params);
  }

  @override
  Future<ResultCode> setParameters(String param) {
    return _invokeMethod('setParameters', {'param': param});
  }

  @override
  Future<RtcVideoStreamManager> videoStreamManager() async {
    if (_videoStreamManager != null) return _videoStreamManager;
    await _invokeMethod('videoStreamManager');
    _videoStreamManager = RtcVideoStreamManager();
    return _videoStreamManager;
  }

  @override
  Future<RtcAnnotationManager> annotationManager() async {
    if (_annotationManager != null) return _annotationManager;
    await _invokeMethod('annotationManager');
    _annotationManager = RtcAnnotationManager();
    return _annotationManager;
  }

  @override
  Future<RtcNetworkManager> networkManager() async {
    if (_networkManager != null) return _networkManager;
    await _invokeMethod('networkManager');
    _networkManager = RtcNetworkManager();
    return _networkManager;
  }
}

mixin RtcEngineKitInterface
    implements
        RtcDeviceManagerInterface,
        RtcAudioMixingManagerInterface,
        RtcSnapshotInterface,
        RtcWhiteboardManagerInterface,
        RtcTroubleshootInterface,
        RtcOptionInterface,
        RtcCustomizedInterface,
        RtcManagersInterface {
  /// @~english
  /// @brief Update the configuration of the RtcEngineKit object.
  /// @param config  RtcEngineConfig object.
  /// @return
  ///   - ResultCode.OK: Success
  ///   - others: Failure
  /// @note Please update the configuration before joining the channel, otherwise it will fail.
  /// @~chinese
  /// @brief 更新 RtcEngineKit 对象配置。
  /// @param config  RtcEngineConfig 对象.
  /// @return
  ///   - ResultCode.OK: 成功
  ///   - 其他: 失败
  /// @note 请在加入频道之前更新配置，否则将返回失败。
  Future<ResultCode> updateConfig(RtcEngineConfig config);

  /// @~english
  /// @brief Destroy the PanoRtcEngineKit object.
  /// @note After destroying the PanoRtcEngineKit object, the object will no longer be valid.
  /// @~chinese
  /// @brief 销毁 PanoRtcEngineKit 对象。
  /// @note PanoRtcEngineKit 对象在销毁后将不再有效。
  Future<void> destroy();

  /// @~english
  /// @brief Join the channel.
  /// @param token     The secure token that App Server got from PANO.
  /// @param channelId The channel ID defined by customer. It must compliance with the following rules:
  ///                  - max length is 128 bytes.
  ///                  - characters can be "0-9", "a-z", "A-Z",
  ///                    whitespace (cannot at leading and trailing),
  ///                    "!", "#", "$", "%", "&", "(", ")", "+", ",", "-", ".", ":",
  ///                    ";", "<", "=", ">", "?", "@", "[", "]", "^", "_", "|", "~".
  /// @param userId    The user ID defined by customer. It must be unique.
  /// @param config    Channel configurations. (Optional)
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Join only one channel at a time.
  ///       User need to check the callback onUserJoinIndication to see if join channel result.
  /// @~chinese
  /// @brief 加入频道。
  /// @param token     应用服务器从PANO获得的安全令牌。
  /// @param channelId 客户定义的频道标识。必须符合以下规则:
  ///                  - 最大长度是128字节；
  ///                  - 只能由以下字符构成：
  ///                    "0-9", "a-z", "A-Z", 空格 (不能出现在首部和尾部),
  ///                    "!", "#", "$", "%", "&", "(", ")", "+", ",", "-", ".", ":",
  ///                    ";", "<", "=", ">", "?", "@", "[", "]", "^", "_", "|", "~"。
  /// @param userId    客户定义的用户标识。必须是唯一的。
  /// @param config    频道设置。（可选）
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 同一时刻只能加入一个频道。
  ///       用户需检查回调函数 onUserJoinIndication 获知加会结果。
  Future<ResultCode> joinChannel(String token, String channelId, int userId,
      {RtcChannelConfig config});

  /// @~english
  /// @brief Leave the channel.
  /// @~chinese
  /// @brief 离开频道。
  Future<void> leaveChannel();

  /// @~english
  /// @brief Start audio.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please join one channel before starting audio, otherwise it will fail.
  /// @~chinese
  /// @brief 开启音频。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 开启音频前请先加入一个频道，否则将返回失败。
  Future<ResultCode> startAudio();

  /// @~english
  /// @brief Stop audio.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 停止音频。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> stopAudio();

  /// @~english
  /// @brief Start video (with a render view).
  /// @param view    PanoView object.
  /// @param config  PanoRtcRenderConfig object.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please join one channel before starting video, otherwise it will fail.
  ///       Must be called from main thread.
  /// @~chinese
  /// @brief 开启视频（随带渲染视图）。
  /// @param view    PanoView 对象。
  /// @param config  PanoRtcRenderConfig 对象。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 开启视频前请先加入一个频道，否则将返回失败。
  ///       必须从主线程调用。
  Future<ResultCode> startVideo(RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config});

  /// @~english
  /// @brief Subscribe to a user's audio.
  /// @param userId  The user ID defined by customer.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Before subscribing to a user’s audio, please make sure the user has started the audio.
  /// @~chinese
  /// @brief 订阅用户的音频。
  /// @param userId  客户定义的用户标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 订阅用户的音频前，请确保用户已开启音频。
  Future<ResultCode> subscribeAudio(int userId);

  /// @~english
  /// @brief Unsubscribe to a user's audio.
  /// @param userId  The user ID defined by customer.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note When a user stops the audio or leaves the channel, the user's audio will be automatically unsubscribed.
  /// @~chinese
  /// @brief 取消订阅用户的音频。
  /// @param userId  客户定义的用户标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 当用户停止音频或者离开频道的时候，用户的音频将会被自动取消订阅。
  Future<ResultCode> unsubscribeAudio(int userId);

  /// @~english
  /// @brief Subscribe to a user's video (with a render view).
  /// @param userId  The user ID defined by customer.
  /// @param view    PanoView object.
  /// @param config  PanoRtcRenderConfig object.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Before subscribing to a user’s video, please make sure the user has started the video.
  ///       Must be called from main thread.
  /// @~chinese
  /// @brief 订阅用户的视频（随带渲染视图）。
  /// @param userId  客户定义的用户标识。
  /// @param view    PanoView 对象。
  /// @param config  PanoRtcRenderConfig 对象。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 订阅用户的视频前，请确保用户已开启视频。
  ///       必须从主线程调用。
  Future<ResultCode> subscribeVideo(int userId, RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config});

  /// @~english
  /// @brief Unsubscribe to a user's video.
  /// @param userId  The user ID defined by customer.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note When a user stops the video or leaves the channel, the user's video will be automatically unsubscribed.
  /// @~chinese
  /// @brief 取消订阅用户的视频。
  /// @param userId  客户定义的用户标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 当用户停止视频或者离开频道的时候，用户的视频将会被自动取消订阅。
  Future<ResultCode> unsubscribeVideo(int userId);

  /// @~english
  /// @brief Subscribe to a user's screen sharing (with a render view).
  /// @param userId  The user ID defined by customer.
  /// @param view    PanoView object.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Before subscribing to a user’s screen sharing, please make sure the user has started the screen sharing.
  ///       Must be called from main thread.
  /// @~chinese
  /// @brief 订阅用户的屏幕共享（随带渲染视图）。
  /// @param userId  客户定义的用户标识。
  /// @param view    PanoView 对象。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 订阅用户的屏幕共享前，请确保用户已开启屏幕共享。
  ///       必须从主线程调用。
  Future<ResultCode> subscribeScreen(int userId, RtcSurfaceViewModel viewModel);

  /// @~english
  /// @brief Unsubscribe to a user's screen sharing.
  /// @param userId  The user ID defined by customer.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note When a user stops the screen sharing or leaves the channel,
  ///       the user's screen sharing will be automatically unsubscribed.
  /// @~chinese
  /// @brief 取消订阅用户的屏幕共享。
  /// @param userId  客户定义的用户标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 当用户停止屏幕共享或者离开频道的时候，用户的屏幕共享将会被自动取消订阅。
  Future<ResultCode> unsubscribeScreen(int userId);

  /// @~english
  /// @brief Update screen absolute scaling ratio.
  /// @param userId  The user ID defined by customer.
  /// @param ratio   Screen scaling ratio type.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note The default focus is center point of view.
  /// @~chinese
  /// @brief 更新屏幕的绝对缩放比例。
  /// @param userId  客户定义的用户标识。
  /// @param ratio   屏幕缩放比例类型。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 默认焦点是视图的中心点。
  Future<ResultCode> updateScreenScaling(int userId, ScreenScalingRatio ratio);

  /// @~english
  /// @brief Update screen relative scaling ratio.
  /// @param userId  The user ID defined by customer.
  /// @param ratio   Screen scaling ratio value.
  /// @param focus   Screen focus coordinate value.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note
  /// @~chinese
  /// @brief 更新屏幕的相对缩放比例。
  /// @param userId  客户定义的用户标识。
  /// @param ratio   屏幕缩放比例值。
  /// @param focus   屏幕焦点坐标值。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note
  Future<ResultCode> updateScreenScalingWithFocus(
      int userId, double ratio, Point<int> focus);

  /// @~english
  /// @brief Update screen relative moving distance.
  /// @param userId    The user ID defined by customer.
  /// @param distance  Screen focus moving distance.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note
  /// @~chinese
  /// @brief 更新屏幕的相对移动距离。
  /// @param userId    客户定义的用户标识。
  /// @param distance  屏幕焦点移动距离。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note
  Future<ResultCode> updateScreenMoving(int userId, Point<int> distance);

  /// @~english
  /// @brief Mute audio.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please start the audio before muting, otherwise it will not work.
  /// @~chinese
  /// @brief 静音。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 静音前请先开启音频，否则操作将无效。
  Future<ResultCode> muteAudio();

  /// @~english
  /// @brief Unmute audio.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please start the audio before unmuting, otherwise it will not work.
  /// @~chinese
  /// @brief 取消静音。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 取消静音前请先开启音频，否则操作将无效。
  Future<ResultCode> unmuteAudio();

  /// @~english
  /// @brief Pause video.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please start the video before pausing, otherwise it will not work.
  /// @~chinese
  /// @brief 暂停视频。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 暂停视频前请先开启视频，否则操作将无效。
  Future<ResultCode> muteVideo();

  /// @~english
  /// @brief Resume video.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Please start the video before resuming, otherwise it will not work.
  /// @~chinese
  /// @brief 恢复视频。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 恢复视频前请先开启视频，否则操作将无效。
  Future<ResultCode> unmuteVideo();
}

mixin RtcDeviceManagerInterface {
  /// @~english
  /// @brief set microphone mute enable status
  /// @param enable  enable mute flag, true/false to mute/unmute
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置麦克风采集数据静音状态(不包括伴音等其他声音)
  /// @param enable  静音开关， 打开/关闭 静音
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setMicrophoneMuteStatus(bool enable);

  /// @~english
  /// @brief Set the volume of the current audio device.
  /// @param volume  Valid value range: [0, 255].
  /// @param type    Device type, PanoDeviceType enum type.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置当前音频设备的音量。
  /// @param volume  有效值范围: [0, 255]。
  /// @param type    设备类型，PanoDeviceType 枚举类型。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setAudioDeviceVolume(int volume, AudioDeviceType type);

  /// @~english
  /// @brief Get the volume of the current audio device.
  /// @param type  Device type, PanoDeviceType enum type.
  /// @return Current volume. Valid value range: [0, 255].
  /// @~chinese
  /// @brief 获取当前音频设备的音量。
  /// @param type  设备类型，PanoDeviceType 枚举类型。
  /// @return 当前音量。有效值范围: [0, 255]。
  Future<int> getAudioDeviceVolume(AudioDeviceType type);

  /// @~english
  /// @brief Get audio capture level.
  /// @return Audio capture level.
  /// @~chinese
  /// @brief 获取音频采集强度值。
  /// @return 音频采集强度值。
  Future<int> getRecordingLevel();

  /// @~english
  /// @brief Get audio playout level.
  /// @return Audio playout level.
  /// @~chinese
  /// @brief 获取音频播放强度值。
  /// @return 音频播放强度值。
  Future<int> getPlayoutLevel();

  /// @~english
  /// @brief Set loudspeaker enable status.
  /// @param enable  Whether to enable.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置扬声器启用状态。
  /// @param enable  是否启用。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setLoudspeakerStatus(bool enable);

  /// @~english
  /// @brief Get loudspeaker enable status.
  /// @return Whether to enable.
  /// @~chinese
  /// @brief 获取扬声器启用状态。
  /// @return 是否启用。
  Future<bool> isEnabledLoudspeaker();

  /// @~english
  /// @brief Switch front and rear cameras.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 切换前后置摄像头。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> switchCamera();

  /// @~english
  /// @brief Get current camera type.
  /// @return Whether it is a front camera.
  /// @~chinese
  /// @brief 获取当前摄像头类型。
  /// @return 是否是前置摄像头。
  Future<bool> isFrontCamera();

  /// @~english
  /// @brief Get camera device ID.
  /// @param frontCamera   Camera device type, YES is front camera, NO is back camera.
  /// @return Device unique ID.
  /// @~chinese
  /// @brief 获取摄像头设备标识。
  /// @param frontCamera   摄像头设备类型，YES是前置摄像头，NO是后置摄像头。
  /// @return 设备唯一标识。
  Future<String> getCameraDeviceId(bool frontCamera);

  /// @~english
  /// @brief Start current camera preview (with a render view).
  /// @param view    PanoView object.
  /// @param config  PanoRtcRenderConfig object.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note Must be called from main thread.
  /// @~chinese
  /// @brief 开启当前摄像头预览（随带渲染视图）。
  /// @param view    PanoView 对象。
  /// @param config  PanoRtcRenderConfig 对象。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 必须从主线程调用。
  Future<ResultCode> startPreview(RtcSurfaceViewModel viewModel,
      {RtcRenderConfig config});

  /// @~english
  /// @brief Stop current camera preview.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 停止当前摄像头预览。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> stopPreview();
}

mixin RtcAudioMixingManagerInterface {
  /// @~english
  /// @brief Create audio mixing task.
  /// @param taskId    unique identifier of task.
  /// @param filename  full path of music file. support mp3, aac, wav.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 创建混音任务。
  /// @param taskId    任务标识。
  /// @param filename  音频文件的完整路径。支持mp3，aac，wav。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> createAudioMixingTask(int taskId, String filename);

  /// @~english
  /// @brief Destroy audio mixing task.
  /// @param taskId    unique identifier of task.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 销毁混音任务。
  /// @param taskId    任务标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> destroyAudioMixingTask(int taskId);

  /// @~english
  /// @brief Start audio mixing task.
  /// @param taskId   unique identifier of task.
  /// @param config   task configuration
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note The real mixing process will only work after joining channel.
  ///
  /// @~chinese
  /// @brief 启动混音任务。
  /// @param taskId    任务标识。
  /// @param config    配置参数。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note 实际的混音操作仅在加入房间后进行。
  Future<ResultCode> startAudioMixingTask(
      int taskId, RtcAudioMixingConfig config);

  /// @~english
  /// @brief Update audio mixing task configuration.
  /// @param taskId   unique identifier of task.
  /// @param config   task configuration
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 更新混音任务配置参数。
  /// @param taskId    任务标识。
  /// @param config    配置参数。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> updateAudioMixingTask(
      int taskId, RtcAudioMixingConfig config);

  /// @~english
  /// @brief Stop audio mixing task.
  /// @param taskId    unique identifier of task.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 结束混音任务。
  /// @param taskId    任务标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> stopAudioMixingTask(int taskId);

  /// @~english
  /// @brief Resume the paused audio mixing task.
  /// @param taskId    unique identifier of task.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 恢复被暂停的混音任务。
  /// @param taskId    任务标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> resumeAudioMixing(int taskId);

  /// @~english
  /// @brief Pause audio mixing task.
  /// @param taskId    unique identifier of task.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 暂停混音任务。
  /// @param taskId    任务标识。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> pauseAudioMixing(int taskId);

  /// @~english
  /// @brief Get duration of music file.
  /// @param taskId    unique identifier of task.
  /// @return Duration with millisecond. If fail, the duration is less than 0.
  /// @note The duration is estimated based on the average bitrate. For some audio files with
  /// a non-constant bitrate, there may be a deviation from the actual value.
  ///
  /// @~chinese
  /// @brief 获取音频文件的总时长。
  /// @param taskId    任务标识。
  /// @return 毫秒级总时长。如果失败，返回值小于0。
  /// @note 总时长是根据文件平均码率估算出来的。对于某些非恒定码率的音频文件，可能与实际总时长相比存在一定偏差。
  Future<int> getAudioMixingDuration(int taskId);

  /// @~english
  /// @brief Get current timestamp.
  /// @param taskId    unique identifier of task.
  /// @return Current timestamp with millisecond. If fail or task has stopped, the timestamp is less than 0.
  ///
  /// @~chinese
  /// @brief 获取当前时间戳。
  /// @param taskId    任务标识。
  /// @return 毫秒级当前时间戳。如果失败或者混音任务已结束，返回值小于0。
  Future<int> getAudioMixingCurrentTimestamp(int taskId);

  /// @~english
  /// @brief Seek to target timestamp.
  /// @param taskId        unique identifier of task.
  /// @param timestampMs   timestamp.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  ///
  /// @~chinese
  /// @brief 跳转至指定时间戳。
  /// @param taskId        任务标识。
  /// @param timestampMs   时间戳。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> seekAudioMixing(int taskId, int timestampMs);
}

mixin RtcSnapshotInterface {
  /// @brief Capture specific user's video content
  /// @param outputDir output directory
  /// @param userId    the id of target user
  /// @param option    snapshot option
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 捕获指定用户的视频画面
  /// @param outputDir 输出路径
  /// @param userId    目标用户的ID
  /// @param option    快照选项
  /// @return
  ///  - OK: 成功
  ///  - 其他: 失败
  Future<ResultCode> snapshotVideo(String outputDir, int userId,
      {RtcSnapshotVideoOption option});
}

mixin RtcWhiteboardManagerInterface {
  /// @~english
  /// @brief Get whiteboard control object.
  /// @return PanoRtcWhiteboard object.
  /// @~chinese
  /// @brief 获取白板控制对象。
  /// @return PanoRtcWhiteboard 对象。
  Future<RtcWhiteboard> whiteboardEngine();

  /// @~english
  /// @brief Switch whiteboard control object.
  /// @param whiteboardId whiteboard Id
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note
  ///   - This interface is used for multiple whiteboard case.
  ///   - PanoRtcEngineKit creates whiteboard with whiteboard Id "default" automatically
  ///   - RtcEngine will reserve whiteboard Id with "pano-" prefix, please don't use it.
  ///   - When call this interface with whiteboard Id not set before, PanoRtcEngineKit will create new whiteboard internal.
  ///   - Call whiteboardEngine to get current whiteboard object after switch.
  /// @~chinese
  /// @brief 切换白板控制对象
  /// @param whiteboardId 白板Id
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note
  ///   - 此接口用于多白板用例场景。
  ///   - PanoRtcEngineKit会自动创建白板Id为"default"的白板
  ///   - RtcEngine会保留前缀为"pano-"的白板Id，请不要使用
  ///   - 当传入的whiteboardId之前没被设置过，PanoRtcEngineKit会生成新的白板
  ///   - 切换后需要调用whiteboardEngine获得当前的白板控制对象。
  Future<ResultCode> switchWhiteboardEngine(String whiteboardId);
}

mixin RtcTroubleshootInterface {
  /// @~english
  /// @brief Start audio dump.
  /// @param filePath      The dump file path.
  /// @param maxFileSize   The max dump file size. If the value is -1, the file size is unlimited.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 开启音频转储。
  /// @param filePath      转储文件路径.
  /// @param maxFileSize   最大转储文件大小. 如果值为-1，则文件大小不受限制。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> startAudioDumpWithFilePath(
      String filePath, int maxFileSize);

  /// @~english
  /// @brief Stop audio dump.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 停止音频转储。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> stopAudioDump();

  /// @~english
  /// @brief Send feedback to PANO.
  /// @param info  Feedback info.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 发送用户反馈到 PANO。
  /// @param info  反馈的信息。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> sendFeedback(FeedbackInfo info);
}

mixin RtcOptionInterface {
  /// @~english
  /// @brief Set option object to PANO SDK.
  /// @param option  The Option object.
  /// @param type    PanoOptionType enum type.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置选项对象给PANO SDK。
  /// @param option  选项对象。
  /// @param type    PanoOptionType 枚举类型。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setOption(dynamic option, OptionType type);
}

mixin RtcCustomizedInterface {
  /// @~english
  /// @brief Set customized parameters to PANO SDK.
  /// @param param   JSON-format parameters.
  /// @return
  ///   - OK: Success
  ///   - others: Failure
  /// @note
  /// @~chinese
  /// @brief 设置自定义参数给PANO SDK。
  /// @param param   JSON格式参数。
  /// @return
  ///   - OK: 成功
  ///   - 其他: 失败
  /// @note
  Future<ResultCode> setParameters(String param);
}

mixin RtcManagersInterface {
  /// @~english
  /// @brief Get video stream manager object.
  /// @return PanoRtcVideoStreamManager object.
  /// @~chinese
  /// @brief 获取视频流管理器对象。
  /// @return PanoRtcVideoStreamManager 对象。
  Future<RtcVideoStreamManager> videoStreamManager();

  /// @~english
  /// @brief Get annotation manager object.
  /// @return PanoRtcAnnotationManager object.
  /// @~chinese
  /// @brief 获取标注管理器对象。
  /// @return PanoRtcAnnotationManager 对象。
  Future<RtcAnnotationManager> annotationManager();

  /// @~english
  /// @brief get the network manager, it can be called before initialize.
  /// @return RtcNetworkManager object.
  /// @~chinese
  /// @brief 获取网络管理器的指针
  /// @return RtcNetworkManager 对象。
  Future<RtcNetworkManager> networkManager();
}
