import 'dart:typed_data';
import 'dart:ui';

import 'package:pano_rtc/pano_rtc.dart';

import 'enum_converter.dart';
import 'rtc_enums.dart';

typedef EmptyCallback = void Function();
typedef ResultCallback = void Function(ResultCode result);
typedef UserIdCallback = void Function(int userId);

//Engine
typedef OnChannelCountDown = void Function(int remain);
typedef OnUserJoinIndication = void Function(int userId, String userName);
typedef OnUserLeaveIndication = void Function(
    int userId, UserLeaveReason reson);
typedef OnUserAudioSubscribe = void Function(
    int userId, SubscribeResult result);
typedef OnUserVideoStart = void Function(
    int userId, VideoProfileType maxProfile);
typedef OnUserVideoSubscribe = void Function(
    int userId, SubscribeResult result);
typedef OnUserScreenSubscribe = void Function(
    int userId, SubscribeResult result);
typedef OnWhiteboardStartWithId = void Function(String whiteboardId);
typedef OnWhiteboardStopWithId = void Function(String whiteboardId);
typedef OnAudioDeviceStateChanged = void Function(
    String deviceId, AudioDeviceType deviceType, AudioDeviceState deviceState);
typedef OnAudioDefaultDeviceChanged = void Function(
    String deviceId, AudioDeviceType deviceType);
typedef OnVideoDeviceStateChanged = void Function(
    String deviceId, VideoDeviceType deviceType, VideoDeviceState deviceState);
typedef OnVideoCaptureStateChanged = void Function(
    String deviceId, VideoCaptureState state);
typedef OnChannelFailover = void Function(FailoverState state);
typedef OnActiveSpeakerListUpdated = void Function(List<int> userIds);
typedef OnAudioMixingStateChanged = void Function(
    int taskId, AudioMixingState state);
typedef OnVideoSnapshotCompleted = void Function(
    bool succeed, int userId, String fileName);
typedef OnNetworkQuality = void Function(int userIds, QualityRating quality);
typedef OnUserAudioLevel = void Function(RtcAudioLevel level);

typedef RtcVideoSendStatsCallback = void Function(RtcVideoSendStats stats);
typedef RtcVideoRecvStatsCallback = void Function(RtcVideoRecvStats stats);
typedef RtcAudioSendStatsCallback = void Function(RtcAudioSendStats stats);
typedef RtcAudioRecvStatsCallback = void Function(RtcAudioRecvStats stats);
typedef RtcVideoSendBweStatsCallback = void Function(
    RtcVideoSendBweStats stats);
typedef RtcVideoRecvBweStatsCallback = void Function(
    RtcVideoRecvBweStats stats);
typedef RtcSystemStatsCallback = void Function(RtcSystemStats stats);

///@~english
///@brief Callback of RtcEngine,  the callbacks must to set to RtcEngine to get events notification.
///@~chinese
///@brief RtcEngine的回调函数， 在使用RtcEigine之前必须要设置回调以获取事件通知。
class RtcEngineEventHandler {
  /// @~english
  ///Notification of join channel
  ///@param result OK means join channel success, others mean join channel fail.
  ///@~chinese
  ///加入频道的通知
  ///@param result OK 表示加入频道成功， 其他表示加入频道失败。
  ResultCallback onChannelJoinConfirm;

  ///@~english
  ///Notification of leave channel
  ///@param result OK means leave channel normally,  others mans leave channel abnormally.
  ///@~chinese
  ///离开频道的通知
  ///@param result OK 表示正常离开频道， 其他表示非正常离开频道。
  ResultCallback onChannelLeaveIndication;

  ///@~english
  ///Notification of channel count down
  ///@param remain channel remian time in seconds.
  ///@~chinese
  ///频道倒计时通知
  ///@param remain 频道剩余时间，单位：秒。

  OnChannelCountDown onChannelCountDown;

  ///@~english
  ///Notification of other attendee join
  ///@param userId user ID
  ///@param userName user name, can be empty string.
  ///@~chinese
  ///其他人员的加入通知
  ///@param userId 用户ID。
  ///@param userName 用户名字， 可能为空字符串。
  OnUserJoinIndication onUserJoinIndication;

  ///@~english
  ///Notification of other attendee leave
  ///@param userId user ID
  ///@param reason reason of leave. OK means attendee leave normally, others attendee mean leave abnormally.
  ///@~chinese
  ///其他人员离开通知。
  ///@param userId 用户ID。
  ///@param reason 用户离开的原因, OK表示正常离开， 其他为非正常离开。
  OnUserLeaveIndication onUserLeaveIndication;

  ///@~english
  ///Notification of attendee start audio
  ///@param userId user ID
  ///@~chinese
  ///用户打开音频通知
  ///@param userId 用户ID。
  UserIdCallback onUserAudioStart;

  ///@~english
  ///Notification of attendee stop audio
  ///@param userId user ID
  ///@~chinese
  ///用户关闭音频通知
  ///@param userId 用户ID。
  UserIdCallback onUserAudioStop;

  ///@~english
  ///Notification of result to subscribe user audio
  ///@param userId user ID
  ///@param result the result of subscribing
  ///@~chinese
  ///用户音频订阅结果通知
  ///@param userId 用户ID。
  ///@param result 订阅结果。
  OnUserAudioSubscribe onUserAudioSubscribe;

  ///@~english
  ///Notification of attendee start video
  ///@param userId user ID
  ///@param maxProfile user video profile
  ///@~chinese
  ///用户开启视频通知
  ///@param userId 用户ID。
  ///@param maxProfile  用户视频能力
  OnUserVideoStart onUserVideoStart;

  ///@~english
  ///Notification of attendee stop video
  ///@param userId user ID
  ///@~chinese
  ///用户关闭视频通知
  ///@param userId 用户ID。
  UserIdCallback onUserVideoStop;

  ///@~english
  ///Notification of result to subscribe user video
  ///@param userId user ID
  ///@param result the result of subscribing
  ///@~chinese
  ///用户视频订阅结果通知
  ///@param userId 用户ID。
  ///@param result 订阅结果。
  OnUserVideoSubscribe onUserVideoSubscribe;

  ///@~english
  ///Notification of attendee mute audio
  ///@param userId user ID
  ///@~chinese
  ///用户设置静音通知
  ///@param userId 用户ID。
  UserIdCallback onUserAudioMute;

  ///@~english
  ///Notification of attendee unmute audio
  ///@param userId user ID
  ///@~chinese
  ///用户取消静音通知
  ///@param userId 用户ID。
  UserIdCallback onUserAudioUnmute;

  ///@~english
  ///Notification of attendee mute video
  ///@param userId user ID
  ///@~chinese
  ///用户暂停视频通知
  ///@param userId 用户ID。
  UserIdCallback onUserVideoMute;

  ///@~english
  ///Notification of attendee unmute video
  ///@param userId user ID
  ///@~chinese
  ///用户恢复视频通知
  ///@param userId 用户ID。
  UserIdCallback onUserVideoUnmute;

  ///@~english
  ///Notification of attendee start screen share
  ///@param userId user ID
  ///@~chinese
  ///用户开启屏幕共享通知
  ///@param userId 用户ID。
  UserIdCallback onUserScreenStart;

  ///@~english
  ///Notification of attendee stop screen share
  ///@param userId user ID
  ///@~chinese
  ///用户关闭屏幕共享通知
  ///@param userId 用户ID。
  UserIdCallback onUserScreenStop;

  ///@~english
  ///Notification of result to subscribe user screen share
  ///@param userId user ID
  ///@param result the result of subscribing
  ///@~chinese
  ///用户桌面共享订阅结果通知
  ///@param userId 用户ID。
  ///@param result 订阅结果。
  OnUserScreenSubscribe onUserScreenSubscribe;

  ///@~english
  ///Notification of attendee mute screen share
  ///@param userId user ID
  ///@~chinese
  ///用户暂停屏幕共享通知
  ///@param userId 用户ID。
  UserIdCallback onUserScreenMute;

  ///@~english
  ///Notification of attendee unmute screen share
  ///@param userId user ID
  ///@~chinese
  ///用户恢复屏幕共享通知
  ///@param userId 用户ID。
  UserIdCallback onUserScreenUnmute;

  ///@~english
  ///Notification of whiteboard service available
  ///@~chinese
  ///白板服务可用通知
  EmptyCallback onWhiteboardAvailable;

  ///@~english
  ///Notification of whiteboard service unavailable
  ///@~chinese
  ///白板服务不可用通知
  EmptyCallback onWhiteboardUnavailable;

  ///@~english
  ///Notification of default whiteboard start
  ///@~chinese
  ///开始默认共享白板通知
  EmptyCallback onWhiteboardStart;

  ///@~english
  ///Notification of default whiteboard stop
  ///@~chinese
  ///终止默认共享白板通知
  EmptyCallback onWhiteboardStop;

  ///@~english
  ///Notification of whiteboard start
  ///@param whiteboardId whiteboard Id
  ///@~chinese
  ///开始共享白板通知
  ///@param whiteboardId 白板Id
  OnWhiteboardStartWithId onWhiteboardStartWithId;

  ///@~english
  ///Notification of whiteboard stop
  ///@param whiteboardId whiteboard Id
  ///@~chinese
  ///终止共享白板通知
  ///@param whiteboardId 白板Id
  OnWhiteboardStopWithId onWhiteboardStopWithId;

  ///@~english
  ///Notification of first audio packet arrived
  ///@param userId user ID
  ///@~chinese
  ///第一个音频包到达通知
  ///@param userId 用户ID。
  UserIdCallback onFirstAudioDataReceived;

  ///@~english
  ///Notification of first video packet arrived
  ///@param userId user ID
  ///@~chinese
  ///第一个视频包到达通知
  ///@param userId 用户ID。
  UserIdCallback onFirstVideoDataReceived;

  ///@~english
  ///Notification of first screen share packet arrived
  ///@param userId user ID
  ///@~chinese
  ///第一个屏幕共享包到达通知
  ///@param userId 用户ID。
  UserIdCallback onFirstScreenDataReceived;

  ///@~english
  ///Notification of first video frame will be rendered.
  ///@param userId user ID
  ///@~chinese
  ///第一个视频帧渲染通知。
  ///@param userId 用户ID。
  UserIdCallback onFirstVideoFrameRendered;

  ///@~english
  ///Notification of first screen share frame will be rendered.
  ///@param userId user ID
  ///@~chinese
  ///第一个屏幕共享帧渲染通知。
  ///@param userId 用户ID。
  UserIdCallback onFirstScreenFrameRendered;

  ///@~english
  ///Notification of audio device state changed
  ///@param deviceID device ID
  ///@param deviceType device type
  ///@param deviceState device state
  ///@~chinese
  ///音频设备状态变化的通知
  ///@param deviceID 设备ID
  ///@param deviceType 设备类型
  ///@param deviceState 设备状态
  OnAudioDeviceStateChanged onAudioDeviceStateChanged;

  ///Notification of audio system default device changed
  ///@param deviceID device ID
  ///@param deviceType device type
  ///@~chinese
  ///音频系统默认设备变化的通知
  ///@param deviceID 设备ID
  ///@param deviceType 设备类型
  OnAudioDefaultDeviceChanged onAudioDefaultDeviceChanged;

  ///@~english
  ///Notification of video device state changed
  ///@param deviceID device ID
  ///@param deviceType device type
  ///@param deviceState device state
  ///@~chinese
  ///视频设备状态变化的通知
  ///@param deviceID 设备ID
  ///@param deviceType 设备类型
  ///@param deviceState 设备状态
  OnVideoDeviceStateChanged onVideoDeviceStateChanged;

  ///@~english
  ///Notification of video capture state changed.
  ///@param deviceID  The device ID.
  ///@param state     The capture state.
  ///@~chinese
  ///视频采集状态变化的通知。
  ///@param deviceID  设备ID。
  ///@param state     采集状态。
  OnVideoCaptureStateChanged onVideoCaptureStateChanged;

  ///@~english
  ///Notification of channel failover
  ///@param state failover state
  ///@~chinese
  ///频道错误恢复通知
  ///@param state 错误恢复状态
  OnChannelFailover onChannelFailover;

  ///@~english
  ///Notification of active speaker list updating
  ///@param userIds The userId list, sorted by users' audio energy
  ///@param count   The count of userId list
  ///@~chinese
  ///活跃用户列表变更通知
  ///@param userIds 用户 ID 列表, 按声音能量值排序
  ///@param count   用户列表大小
  OnActiveSpeakerListUpdated onActiveSpeakerListUpdated;

  ///@~english
  ///Notification of audio mixing state changing
  ///@param taskId  Unique identifier of task.
  ///@param state   State of Audio mixing task.
  ///@~chinese
  ///混音任务状态变更通知
  ///@param taskId  任务标识
  ///@param state   混音状态
  OnAudioMixingStateChanged onAudioMixingStateChanged;

  ///@~english
  ///Notification of user's video snapshot completed
  ///@param succeed    Succeed to write the image.
  ///@param userId     The userId of snapshot source
  ///@param filename   The snapshot image full path with name.
  ///@~chinese
  ///用户视频快照完成通知
  ///@param succeed    是否成功写入文件
  ///@param userId     快照所属的用户ID
  ///@param filename   快照文件完整路径
  OnVideoSnapshotCompleted onVideoSnapshotCompleted;

  ///@~english
  ///Notification of in-call network quality
  ///@param userId   The user ID
  ///@param quality  The network quality.
  ///@~chinese
  ///通话中的网络质量通知
  ///@param userId   用户ID
  ///@param quality  网络质量
  OnNetworkQuality onNetworkQuality;

  ///@~english
  ///@brief Notification of audio start result
  ///@param result The result of audio start
  ///@~chinese
  ///@brief 音频开启成功与否的通知
  ///@param result 音频开启的结果
  ResultCallback onAudioStartResult;

  ///@~english
  ///@brief Notification of video start result
  ///@param result The result of video start
  ///@~chinese
  ///@brief 视频开启成功与否的通知
  ///@param result 视频开启的结果
  ResultCallback onVideoStartResult;

  ///@~english
  ///@brief Notification of sharing start result
  ///@param result The result of sharing start
  ///@~chinese
  ///@brief 共享开启成功与否的通知
  ///@param result 共享开启的结果
  ResultCallback onScreenStartResult;

  ///@~english
  ///@brief Callback user audio level.
  ///@param level   Current audio level.
  ///@~chinese
  ///@brief 回调用户声音强度。
  ///@param level   当前的用户强度。
  OnUserAudioLevel onUserAudioLevel;

  ///@~english
  ///@brief Callback statistics of sent video.
  ///@param stats   Statistics of sent video.
  ///@~chinese
  ///@brief 回调发送视频的统计。
  ///@param stats   已发送视频的统计信息。
  RtcVideoSendStatsCallback onVideoSendStats;

  ///@~english
  ///@brief Callback statistics of received video.
  ///@param stats   Statistics of received video.
  ///@~chinese
  ///@brief 回调接收视频的统计。
  ///@param stats   已接收视频的统计信息。
  RtcVideoRecvStatsCallback onVideoRecvStats;

  ///@~english
  ///@brief Callback statistics of sent audio.
  ///@param stats   Statistics of sent aduio.
  ///@~chinese
  ///@brief 回调发送音频的统计。
  ///@param stats   已发送音频的统计信息。
  RtcAudioSendStatsCallback onAudioSendStats;

  ///@~english
  ///@brief Callback statistics of received audio.
  ///@param stats   Statistics of received audio.
  ///@~chinese
  ///@brief 回调接收音频的统计。
  ///@param stats   已接收音频的统计信息。
  RtcAudioRecvStatsCallback onAudioRecvStats;

  ///@~english
  ///@brief Callback statistics of sent screen sharing.
  ///@param stats   Statistics of sent video.
  ///@~chinese
  ///@brief 回调发送屏幕共享的统计。
  ///@param stats   已发送屏幕共享的统计信息。
  RtcVideoSendStatsCallback onScreenSendStats;

  ///@~english
  ///@brief Callback statistics of received screen sharing.
  ///@param stats   Statistics of received screen sharing.
  ///@~chinese
  ///@brief 回调接收屏幕共享的统计。
  ///@param stats   已接收屏幕共享的统计信息。
  RtcVideoRecvStatsCallback onScreenRecvStats;

  ///@~english
  ///@brief Callback bandwidth estimation information of sent video.
  ///@param stats   Bandwidth estimation of sent video.
  ///@note Total bandwidth evaluation including sent video and screen sharing.
  ///@~chinese
  ///@brief 回调发送视频的带宽评估。
  ///@param stats   发送视频的带宽评估信息。
  ///@note 包含发送视频和屏幕共享的总共带宽评估。
  RtcVideoSendBweStatsCallback onVideoSendBweStats;

  ///@~english
  ///@brief Callback bandwidth estimation information of received video.
  ///@param stats   Bandwidth estimation of received video.
  ///@note Total bandwidth estimation including received video and screen sharing.
  ///@~chinese
  ///@brief 回调接收视频的带宽评估。
  ///@param stats   接收视频的带宽评估信息。
  ///@note 包含接收视频和屏幕共享的总共带宽评估。
  RtcVideoRecvBweStatsCallback onVideoRecvBweStats;

  ///@~english
  ///@brief Callback system statistics.
  ///@param stats   Current system statistics.
  ///@~chinese
  ///@brief 回调系统统计信息。
  ///@param stats   当前的系统统计信息。
  RtcSystemStatsCallback onSystemStats;

  /// Constructs a [RtcEngineEventHandler]
  RtcEngineEventHandler(
      {this.onChannelJoinConfirm,
      this.onChannelLeaveIndication,
      this.onChannelCountDown,
      this.onUserJoinIndication,
      this.onUserLeaveIndication,
      this.onUserAudioStart,
      this.onUserAudioStop,
      this.onUserAudioSubscribe,
      this.onUserVideoStart,
      this.onUserVideoStop,
      this.onUserVideoSubscribe,
      this.onUserAudioMute,
      this.onUserAudioUnmute,
      this.onUserVideoMute,
      this.onUserVideoUnmute,
      this.onUserScreenStart,
      this.onUserScreenStop,
      this.onUserScreenSubscribe,
      this.onUserScreenMute,
      this.onUserScreenUnmute,
      this.onWhiteboardAvailable,
      this.onWhiteboardUnavailable,
      this.onWhiteboardStart,
      this.onWhiteboardStop,
      this.onWhiteboardStartWithId,
      this.onWhiteboardStopWithId,
      this.onFirstAudioDataReceived,
      this.onFirstVideoDataReceived,
      this.onFirstScreenDataReceived,
      this.onFirstVideoFrameRendered,
      this.onFirstScreenFrameRendered,
      this.onAudioDeviceStateChanged,
      this.onAudioDefaultDeviceChanged,
      this.onVideoDeviceStateChanged,
      this.onVideoCaptureStateChanged,
      this.onChannelFailover,
      this.onActiveSpeakerListUpdated,
      this.onAudioMixingStateChanged,
      this.onVideoSnapshotCompleted,
      this.onNetworkQuality,
      this.onAudioStartResult,
      this.onVideoStartResult,
      this.onScreenStartResult,
      this.onUserAudioLevel,
      this.onVideoSendStats,
      this.onVideoRecvStats,
      this.onAudioSendStats,
      this.onAudioRecvStats,
      this.onScreenSendStats,
      this.onScreenRecvStats,
      this.onVideoSendBweStats,
      this.onVideoRecvBweStats,
      this.onSystemStats});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onChannelJoinConfirm':
        onChannelJoinConfirm?.call(ResultCodeConverter.fromValue(data[0]).e);
        break;
      case 'onChannelLeaveIndication':
        onChannelLeaveIndication
            ?.call(ResultCodeConverter.fromValue(data[0]).e);
        break;
      case 'onChannelCountDown':
        onChannelCountDown?.call(data[0]);
        break;
      case 'onUserJoinIndication':
        onUserJoinIndication?.call(data[0], data[1]);
        break;
      case 'onUserLeaveIndication':
        onUserLeaveIndication?.call(
            data[0], UserLeaveReasonConverter.fromValue(data[1]).e);
        break;
      case 'onUserAudioStart':
        onUserAudioStart?.call(data[0]);
        break;
      case 'onUserAudioStop':
        onUserAudioStop?.call(data[0]);
        break;
      case 'onUserAudioSubscribe':
        onUserAudioSubscribe?.call(
            data[0], SubscribeResultConverter.fromValue(data[1]).e);
        break;
      case 'onUserVideoStart':
        onUserVideoStart?.call(
            data[0], VideoProfileTypeConverter.fromValue(data[1]).e);
        break;
      case 'onUserVideoStop':
        onUserVideoStop?.call(data[0]);
        break;
      case 'onUserVideoSubscribe':
        onUserVideoSubscribe?.call(
            data[0], SubscribeResultConverter.fromValue(data[1]).e);
        break;
      case 'onUserAudioMute':
        onUserAudioMute?.call(data[0]);
        break;
      case 'onUserAudioUnmute':
        onUserAudioUnmute?.call(data[0]);
        break;
      case 'onUserVideoMute':
        onUserVideoMute?.call(data[0]);
        break;
      case 'onUserVideoUnmute':
        onUserVideoUnmute?.call(data[0]);
        break;
      case 'onUserScreenStart':
        onUserScreenStart?.call(data[0]);
        break;
      case 'onUserScreenStop':
        onUserScreenStop?.call(data[0]);
        break;
      case 'onUserScreenSubscribe':
        onUserScreenSubscribe?.call(
            data[0], SubscribeResultConverter.fromValue(data[1]).e);
        break;
      case 'onUserScreenMute':
        onUserScreenMute?.call(data[0]);
        break;
      case 'onUserScreenUnmute':
        onUserScreenUnmute?.call(data[0]);
        break;
      case 'onWhiteboardAvailable':
        onWhiteboardAvailable?.call();
        break;
      case 'onWhiteboardUnavailable':
        onWhiteboardUnavailable?.call();
        break;
      case 'onWhiteboardStart':
        onWhiteboardStart?.call();
        break;
      case 'onWhiteboardStop':
        onWhiteboardStop?.call();
        break;
      case 'onWhiteboardStartWithId':
        onWhiteboardStartWithId?.call(data[0]);
        break;
      case 'onWhiteboardStopWithId':
        onWhiteboardStopWithId?.call(data[0]);
        break;
      case 'onFirstAudioDataReceived':
        onFirstAudioDataReceived?.call(data[0]);
        break;
      case 'onFirstVideoDataReceived':
        onFirstVideoDataReceived?.call(data[0]);
        break;
      case 'onFirstScreenDataReceived':
        onFirstScreenDataReceived?.call(data[0]);
        break;
      case 'onFirstVideoFrameRendered':
        onFirstVideoFrameRendered?.call(data[0]);
        break;
      case 'onFirstScreenFrameRendered':
        onFirstScreenFrameRendered?.call(data[0]);
        break;
      case 'onAudioDeviceStateChanged':
        onAudioDeviceStateChanged?.call(
            data[0],
            AudioDeviceTypeConverter.fromValue(data[1]).e,
            AudioDeviceStateConverter.fromValue(data[2]).e);
        break;
      case 'onAudioDefaultDeviceChanged':
        onAudioDefaultDeviceChanged?.call(
            data[0], AudioDeviceTypeConverter.fromValue(data[1]).e);
        break;
      case 'onVideoDeviceStateChanged':
        onVideoDeviceStateChanged?.call(
            data[0],
            VideoDeviceTypeConverter.fromValue(data[1]).e,
            VideoDeviceStateConverter.fromValue(data[2]).e);
        break;
      case 'onVideoCaptureStateChanged':
        onVideoCaptureStateChanged?.call(
            data[0], VideoCaptureStateConverter.fromValue(data[1]).e);
        break;
      case 'onChannelFailover':
        onChannelFailover?.call(FailoverStateConverter.fromValue(data[0]).e);
        break;
      case 'onActiveSpeakerListUpdated':
        onActiveSpeakerListUpdated?.call(data[0]);
        break;
      case 'onAudioMixingStateChanged':
        onAudioMixingStateChanged?.call(
            data[0], AudioMixingStateConverter.fromValue(data[1]).e);
        break;
      case 'onVideoSnapshotCompleted':
        onVideoSnapshotCompleted?.call(data[0], data[1], data[2]);
        break;
      case 'onNetworkQuality':
        onNetworkQuality?.call(
            data[0], QualityRatingConverter.fromValue(data[1]).e);
        break;
      case 'onAudioStartResult':
        onAudioStartResult?.call(ResultCodeConverter.fromValue(data[0]).e);
        break;
      case 'onVideoStartResult':
        onVideoStartResult?.call(ResultCodeConverter.fromValue(data[0]).e);
        break;
      case 'onScreenStartResult':
        onScreenStartResult?.call(ResultCodeConverter.fromValue(data[0]).e);
        break;
      case 'onUserAudioLevel':
        onUserAudioLevel?.call(RtcAudioLevel.fromJson(data[0]));
        break;
      case 'onVideoSendStats':
        onVideoSendStats.call(RtcVideoSendStats.fromJson(data[0]));
        break;
      case 'onVideoRecvStats':
        onVideoRecvStats.call(RtcVideoRecvStats.fromJson(data[0]));
        break;
      case 'onAudioSendStats':
        onAudioSendStats.call(RtcAudioSendStats.fromJson(data[0]));
        break;
      case 'onAudioRecvStats':
        onAudioRecvStats.call(RtcAudioRecvStats.fromJson(data[0]));
        break;
      case 'onScreenSendStats':
        onScreenSendStats.call(RtcVideoSendStats.fromJson(data[0]));
        break;
      case 'onScreenRecvStats':
        onScreenRecvStats.call(RtcVideoRecvStats.fromJson(data[0]));
        break;
      case 'onVideoSendBweStats':
        onVideoSendBweStats.call(RtcVideoSendBweStats.fromJson(data[0]));
        break;
      case 'onVideoRecvBweStats':
        onVideoRecvBweStats.call(RtcVideoRecvBweStats.fromJson(data[0]));
        break;
      case 'onSystemStats':
        onSystemStats.call(RtcSystemStats.fromJson(data[0]));
        break;
    }
  }
}

//Whiteboard
typedef FileIdCallback = void Function(ResultCode result, String fileId);
typedef OnPageNumberChanged = void Function(int curPage, int totalPages);
typedef OnImageStateChanged = void Function(String url, WBImageState status);
typedef OnViewScaleChanged = void Function(double scale);
typedef OnRoleTypeChanged = void Function(WBRoleType newRole);
typedef OnMessage = void Function(int userId, Uint8List byte);
typedef OnDocTranscodeStatus = void Function(
    ResultCode result, String fileId, int progress, int totalPages);
typedef OnCreateDoc = void Function(
    ResultCode result, String fileId, String name);
typedef OnSaveDoc = void Function(
    ResultCode result, String fileId, String outputDir);
typedef OnDocThumbnailReady = void Function(String fileId, List<String> urls);
typedef OnDrawRectUpdate = void Function(
    double offsetX, double offsetY, Size viewSize, Size maxSize);
typedef OnUserJoined = void Function(int userId, String userName);

///@~english
///@brief Callback of RtcWhiteboard,  the callback must set to RtcWhiteboard to get events notification.
///@~chinese
///@brief RtcWhiteboard 的回调函数， 在使用 RtcWhiteboard 之前必须要设置回调以获取事件通知。
class WhiteboardEventHandler {
  ///@~english
  ///Notification of whiteboard data synced
  ///@~chinese
  ///白板数据同步完成通知
  EmptyCallback onStatusSynced;

  ///@~english
  ///Notification of page number changed
  ///@param curPage new page number.
  ///@~chinese
  ///白板页码变化通知
  ///@param curPage 新当前页码
  OnPageNumberChanged onPageNumberChanged;

  ///@~english
  ///Notification of image state changed
  ///@param pageNo    page number.
  ///@param url       image url.
  ///@param state     image state.
  ///@~chinese
  ///图片状态变化通知
  ///@param pageNo    页码。
  ///@param url       图片 URL。
  ///@param state     图片状态码。
  OnImageStateChanged onImageStateChanged;

  ///@~english
  ///Notification of whiteboard view scale factor changed
  ///@param scale     The scale factor.
  ///@~chinese
  ///白板视图缩放比例变化通知
  ///@param scale     缩放比例。
  OnViewScaleChanged onViewScaleChanged;

  ///@~english
  ///Notification of whiteboard role type changed
  ///@param newRole   The new role type.
  ///@~chinese
  ///白板角色类型变化通知
  ///@param newRole   新角色。
  OnRoleTypeChanged onRoleTypeChanged;

  ///@~english
  ///Notification of whiteboard content update
  ///@~chinese
  ///白板内容更新通知
  EmptyCallback onContentUpdated;

  ///@~english
  ///Notification of whiteboard snapshot complete
  ///@param result     Snapshot result
  ///@param filename   Snapshot image full path with name.
  ///@~chinese
  ///白板快照完成通知
  ///@param result     快照结果
  ///@param filename   快照文件名
  FileIdCallback onSnapshotComplete;

  ///@~english
  ///Notification of message
  ///@param userId The user who send the message
  ///@~chinese
  ///消息通知
  ///@param userId 消息发送者 ID
  OnMessage onMessage;

  ///@~english
  ///@brief Notification of add background images
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@~chinese
  ///@brief 添加一组背景图结果通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  FileIdCallback onAddBackgroundImages;

  ///@~english
  ///@brief Notification of doc transcode status
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@param progress Transcode progress
  ///@param totalPage Transcode total page number
  ///@~chinese
  ///@brief 文档转码状态通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  ///@param progress 转码进度
  ///@param totalPage 转码总页数
  OnDocTranscodeStatus onDocTranscodeStatus;

  ///@~english
  ///@brief Notification of create whiteboard file
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@~chinese
  ///@brief 创建白板文件通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  FileIdCallback onCreateDoc;

  ///@~english
  ///@brief Notification of create whiteboard file
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@param name Whiteboard file name
  ///@~chinese
  ///@brief 创建白板文件通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  ///@param name 白板文件名称
  OnCreateDoc onCreateDocWithName;

  ///@~english
  ///@brief Notification of delete whiteboard file
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@~chinese
  ///@brief 删除白板文件通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  FileIdCallback onDeleteDoc;

  ///@~english
  ///@brief Notification of switch whiteboard file
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@~chinese
  ///@brief 切换白板文件通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  FileIdCallback onSwitchDoc;

  ///@~english
  ///@brief Notification of save whiteboard file
  ///@param result  Notification result
  ///@param fileId Whiteboard file ID
  ///@param outputDir Output directory
  ///@~chinese
  ///@brief 保存白板文件通知
  ///@param result  通知结果
  ///@param fileId 白板文件ID
  ///@param outputDir 输出路径
  OnSaveDoc onSaveDoc;

  ///@~english
  ///@brief Notification of whiteboard file thumbnail ready
  ///@param fileId Whiteboard file ID
  ///@param urls thumbnail url array
  ///@param count thumbnail url array count
  ///@~chinese
  ///@brief 白板文件缩略图完成通知
  ///@param fileId 白板文件ID
  ///@param urls 缩略图url数组
  ///@param count 缩略图url数组大小
  OnDocThumbnailReady onDocThumbnailReady;

  ///@~english
  ///@brief Notification of whiteboard draw area update
  ///@param offsetX view horizontal offset relative to whole whiteboard
  ///@param offsetY view vertical offset relative to whole whiteboard
  ///@param viewSize view size
  ///@param maxSize whole whiteboard size
  ///@~chinese
  ///@brief 白板绘制区域更新通知
  ///@param offsetX  视图区域相对于整个白板区域X方向偏移
  ///@param offsetY  视图区域相对于整个白板区域Y方向偏移
  ///@param viewSize 视图大小
  ///@param maxSize 白板大小
  OnDrawRectUpdate onDrawRectUpdate;

  ///@~english
  ///Notification of attendee join
  ///@param userId user ID
  ///@param userName user name, can be empty string.
  ///@~chinese
  ///人员的加入通知
  ///@param userId 用户ID。
  ///@param userName 用户名字， 可能为空字符串
  OnUserJoined onUserJoined;

  ///@~english
  ///Notification of attendee leave
  ///@param userId user ID
  ///@~chinese
  ///人员离开通知。
  ///@param userId 用户ID。
  UserIdCallback onUserLeft;

  /// Constructs a [WhiteboardEventHandler]
  WhiteboardEventHandler(
      {this.onStatusSynced,
      this.onPageNumberChanged,
      this.onImageStateChanged,
      this.onViewScaleChanged,
      this.onRoleTypeChanged,
      this.onContentUpdated,
      this.onSnapshotComplete,
      this.onMessage,
      this.onAddBackgroundImages,
      this.onDocTranscodeStatus,
      this.onCreateDoc,
      this.onCreateDocWithName,
      this.onDeleteDoc,
      this.onSwitchDoc,
      this.onSaveDoc,
      this.onDocThumbnailReady,
      this.onUserJoined,
      this.onUserLeft});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onStatusSynced':
        onStatusSynced?.call();
        break;
      case 'onPageNumberChanged':
        onPageNumberChanged?.call(data[0], data[1]);
        break;
      case 'onImageStateChanged':
        onImageStateChanged?.call(
            data[0], WBImageStateConverter.fromValue(data[1]).e);
        break;
      case 'onViewScaleChanged':
        onViewScaleChanged?.call(data[0]);
        break;
      case 'onRoleTypeChanged':
        onRoleTypeChanged?.call(WBRoleTypeConverter.fromValue(data[0]).e);
        break;
      case 'onContentUpdated':
        onContentUpdated?.call();
        break;
      case 'onSnapshotComplete':
        onSnapshotComplete?.call(
            ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
      case 'onMessage':
        onMessage?.call(data[0], data[1]);
        break;
      case 'onAddBackgroundImages':
        onAddBackgroundImages?.call(
            ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
      case 'onDocTranscodeStatus':
        onDocTranscodeStatus?.call(ResultCodeConverter.fromValue(data[0]).e,
            data[1], data[2], data[3]);
        break;
      case 'onCreateDoc':
        onCreateDoc?.call(ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
      case 'onCreateDocWithName':
        onCreateDocWithName?.call(
            ResultCodeConverter.fromValue(data[0]).e, data[1], data[2]);
        break;
      case 'onDeleteDoc':
        onDeleteDoc?.call(ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
      case 'onSwitchDoc':
        onSwitchDoc?.call(ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
      case 'onSaveDoc':
        onSaveDoc?.call(
            ResultCodeConverter.fromValue(data[0]).e, data[1], data[2]);
        break;
      case 'onDocThumbnailReady':
        onDocThumbnailReady?.call(data[0], List<String>.from(data[1]));
        break;
      case 'onDrawRectUpdate':
        onDrawRectUpdate?.call(
            data[0], data[1], Size(data[2], data[3]), Size(data[4], data[5]));
        break;
      case 'onUserJoined':
        onUserJoined?.call(data[0], data[1]);
        break;
      case 'onUserLeft':
        onUserLeft?.call(data[0]);
        break;
    }
  }
}

//AnnotationManager
typedef OnVideoAnnotationStart = void Function(int userId, int streamId);
typedef OnVideoAnnotationStop = void Function(int userId, int streamId);
typedef OnUserVideoMute = void Function(int userId, int streamId);

///@~english
///@brief Annotation Manager
///@~chinese
///@brief 标注管理器
class AnnotationMgrEventHandler {
  ///@~english
  ///Notification of video annotation start
  ///@param userId User ID
  ///@param streamId Stream ID
  ///@~chinese
  ///开始视频标注通知
  ///@param userId 用户ID
  ///@param streamId 视频流ID
  OnVideoAnnotationStart onVideoAnnotationStart;

  ///@~english
  ///Notification of video annotation stop
  ///@param userId User ID
  ///@param streamId Stream ID
  ///@~chinese
  ///终止视频标注通知
  ///@param userId 用户ID
  ///@param streamId 视频流ID
  OnVideoAnnotationStop onVideoAnnotationStop;

  ///@~english
  ///Notification of share annotation start
  ///@param userId User ID
  ///@~chinese
  ///开始共享标注通知
  ///@param userId 用户ID
  UserIdCallback onShareAnnotationStart;

  ///@~english
  ///Notification of share annotation stop
  ///@param userId User ID
  ///@~chinese
  ///终止共享标注通知
  ///@param userId 用户ID
  UserIdCallback onShareAnnotationStop;

  /// Constructs a [AnnotationMgrEventHandler]
  AnnotationMgrEventHandler(
      {this.onVideoAnnotationStart,
      this.onVideoAnnotationStop,
      this.onShareAnnotationStart,
      this.onShareAnnotationStop});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onVideoAnnotationStart':
        onVideoAnnotationStart?.call(data[0], data[1]);
        break;
      case 'onVideoAnnotationStop':
        onVideoAnnotationStop?.call(data[0], data[1]);
        break;
      case 'onShareAnnotationStart':
        onShareAnnotationStart?.call(data[0]);
        break;
      case 'onShareAnnotationStop':
        onShareAnnotationStop?.call(data[0]);
        break;
    }
  }
}

typedef OnAnnoRoleChanged = void Function(WBRoleType newRole);
typedef OnSnapshotComplete = void Function(ResultCode result, String filename);

///@~english
///@brief Callback of PanoAnnotation,  the callback must set to PanoAnnotation to get events notification.
///@~chinese
///@brief PanoAnnotation 的回调函数， 在使用 PanoAnnotation 之前必须要设置回调以获取事件通知。
class AnnotationEventHandler {
  ///@~english
  ///Notification of annotation role type changed
  ///@param newRole   The new role type.
  ///@~chinese
  ///标注角色类型变化通知
  ///@param newRole   新角色。
  OnAnnoRoleChanged onAnnoRoleChanged;

  ///@~english
  ///Notification of annotation snapshot complete
  ///@param result     Snapshot result
  ///@param filename   Snapshot image full path with name.
  ///@~chinese
  ///标注快照完成通知
  ///@param result     快照结果
  ///@param filename   快照文件名
  OnSnapshotComplete onSnapshotComplete;

  /// Constructs a [AnnotationEventHandler]
  AnnotationEventHandler({this.onAnnoRoleChanged, this.onSnapshotComplete});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onAnnoRoleChanged':
        onAnnoRoleChanged?.call(WBRoleTypeConverter.fromValue(data[0]).e);
        break;
      case 'onSnapshotComplete':
        onSnapshotComplete?.call(
            ResultCodeConverter.fromValue(data[0]).e, data[1]);
        break;
    }
  }
}

//VideoStreamManager
typedef UserStreamCallback = void Function(int userId, int streamId);
typedef OnUserVideoStreamStart = void Function(
    int userId, int streamId, VideoProfileType maxProfile);
typedef OnUserVideoStreamSubscribe = void Function(
    int userId, int streamId, SubscribeResult result);
typedef OnVideoStreamSnapshotCompleted = void Function(
    int userId, int streamId, bool succeed, String filename);
typedef OnVideoStreamCaptureStateChanged = void Function(
    int streamId, String deviceId, VideoCaptureState state);
typedef OnVideoStartResult = void Function(int streamId, ResultCode result);

///@~english
///@brief Callback of VideoStreamManager, the callback must be set to stream manager to get events notification.
///@~chinese
///@brief VideoStreamManager 的回调函数， 在使用 VideoStreamManager 之前必须要设置回调以获取事件通知。
class VideoStreamEventHandler {
  ///@~english
  ///Notification of attendee start video stream
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@param maxProfile user video profile
  ///@~chinese
  ///用户开启视频流通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  ///@param maxProfile  用户视频能力
  OnUserVideoStreamStart onUserVideoStreamStart;

  ///@~english
  ///Notification of attendee stop video stream
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///用户关闭视频流通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  UserStreamCallback onUserVideoStreamStop;

  ///@~english
  ///Notification of result to subscribe user video stream
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@param result the result of subscribing
  ///@~chinese
  ///用户视频流订阅结果通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  ///@param result 订阅结果。
  OnUserVideoStreamSubscribe onUserVideoStreamSubscribe;

  ///@~english
  ///Notification of attendee mute video stream
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///用户暂停视频流通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  UserStreamCallback onUserVideoMute;

  ///@~english
  ///Notification of attendee unmute video stream
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///用户恢复视频流通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  UserStreamCallback onUserVideoUnmute;

  ///@~english
  ///Notification of first video packet arrived
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///第一个视频包到达通知
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  UserStreamCallback onFirstVideoDataReceived;

  ///@~english
  ///Notification of first video frame will be rendered.
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///第一个视频帧渲染通知。
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  UserStreamCallback onFirstVideoFrameRendered;

  ///@~english
  ///Notification of first video frame will be rendered.
  ///@param userId user ID
  ///@param streamId video stream ID
  ///@~chinese
  ///第一个视频帧渲染通知。
  ///@param userId 用户ID。
  ///@param streamId 视频流 ID。
  OnVideoStreamSnapshotCompleted onVideoStreamSnapshotCompleted;

  ///@~english
  ///Notification of user's video snapshot completed
  ///@param userId     The userId of snapshot source
  ///@param streamId   The video stream ID
  ///@param succeed    Succeed to write the image.
  ///@param filename   The snapshot image full path with name.
  ///@~chinese
  ///用户视频快照完成通知
  ///@param userId     快照所属的用户ID
  ///@param streamId   视频流 ID。
  ///@param succeed    是否成功写入文件
  ///@param filename   快照文件完整路径
  OnVideoStreamCaptureStateChanged onVideoCaptureStateChanged;

  ///@~english
  ///@brief Notification of video stream start result
  ///@param streamId Stream ID.
  ///@param result The result of video stream start
  ///@~chinese
  ///@brief 视频流开启成功与否的通知
  ///@param streamId 视频流 ID
  ///@param result 视频流开启的结果
  OnVideoStartResult onVideoStartResult;

  /// Constructs a [VideoStreamEventHandler]
  VideoStreamEventHandler(
      {this.onUserVideoStreamStart,
      this.onUserVideoStreamStop,
      this.onUserVideoStreamSubscribe,
      this.onUserVideoMute,
      this.onUserVideoUnmute,
      this.onFirstVideoDataReceived,
      this.onFirstVideoFrameRendered,
      this.onVideoStreamSnapshotCompleted,
      this.onVideoCaptureStateChanged,
      this.onVideoStartResult});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onUserVideoStreamStart':
        onUserVideoStreamStart?.call(
            data[0], data[1], VideoProfileTypeConverter.fromValue(data[2]).e);
        break;
      case 'onUserVideoStreamStop':
        onUserVideoStreamStop?.call(data[0], data[1]);
        break;
      case 'onUserVideoStreamSubscribe':
        onUserVideoStreamSubscribe?.call(
            data[0], data[1], SubscribeResultConverter.fromValue(data[2]).e);
        break;
      case 'onUserVideoMute':
        onUserVideoMute?.call(data[0], data[1]);
        break;
      case 'onUserVideoUnmute':
        onUserVideoUnmute?.call(data[0], data[1]);
        break;
      case 'onFirstVideoDataReceived':
        onFirstVideoDataReceived?.call(data[0], data[1]);
        break;
      case 'onFirstVideoFrameRendered':
        onFirstVideoFrameRendered?.call(data[0], data[1]);
        break;
      case 'onVideoStreamSnapshotCompleted':
        onVideoStreamSnapshotCompleted?.call(
            data[0], data[1], data[2], data[3]);
        break;
      case 'onVideoCaptureStateChanged':
        onVideoCaptureStateChanged?.call(
            data[0], data[1], VideoCaptureStateConverter.fromValue(data[2]).e);
        break;
      case 'onVideoStartResult':
        onVideoStartResult?.call(
            data[0], ResultCodeConverter.fromValue(data[1]).e);
        break;
    }
  }
}

typedef OnNetworkTestComplete = void Function(RtcNetworkQuality quality);

class RtcNetworkMgrHandler {
  ///@~english
  ///@brief Callback the result of network test.
  ///@param quality Network quality report
  ///@~chinese
  ///@brief 回调网络检测的结果。
  ///@param quality 网络质量报告
  OnNetworkTestComplete onNetworkTestComplete;

  /// Constructs a [RtcNetworkMgrHandler]
  RtcNetworkMgrHandler({this.onNetworkTestComplete});

  void process(String methodName, List<dynamic> data) {
    switch (methodName) {
      case 'onNetworkTestComplete':
        onNetworkTestComplete?.call(RtcNetworkQuality.fromJson(data[0]));
        break;
    }
  }
}
