import 'package:flutter/services.dart';

import '../pano_rtc.dart';

class RtcNetworkManager with RtcNetworkManagerInterface {
  final MethodChannel _methodChannel = MethodChannel('pano_rtc/api_networkMgr');
  final EventChannel _eventChannel = EventChannel('pano_rtc/events_networkMgr');
  RtcNetworkMgrHandler _handler;

  RtcNetworkManager() {
    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  void setEventHandler(RtcNetworkMgrHandler handler) {
    _handler = handler;
  }

  @override
  Future<ResultCode> startNetworkTest(String token) {
    return _methodChannel.invokeMethod('startNetworkTest', {'token': token});
  }

  @override
  Future<ResultCode> stopNetworkTest() {
    return _methodChannel.invokeMethod('stopNetworkTest');
  }
}

mixin RtcNetworkManagerInterface {
  ///@~english
  ///@brief start test network.
  ///@param token   the secure token that App Server got from PANO
  ///               the channelId can be PANO_NHC_NetworkTest when generating this token
  ///               NOTE: don't use the token which is used for joining RtcChannel
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note The RtcEngine must be initialized firstly before startNetworkTest
  ///      The network test will consume additional bandwidth, please avoid doing network test in call
  ///@~chinese
  ///@brief 启动网络测试。
  ///@param token   App服务器向PANO获取的token。生成此 token 时 channelId 可为 PANO_NHC_NetworkTest
  ///               注意：不要使用加入 RtcChannel 时使用的 token
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 在启动 startNetworkTest 前必须先初始化 RtcEngine
  ///      网络测试会产生额外流量，尽量避免在通话过程中进行测试。
  Future<ResultCode> startNetworkTest(String token);

  ///@~english
  ///@brief stop test network.
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@~chinese
  ///@brief 停止网络测试。
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  Future<ResultCode> stopNetworkTest();
}


