import 'package:json_annotation/json_annotation.dart';

import 'rtc_enums.dart';

part 'rtc_objects.g.dart';

/// @brief @~english The configurations class of the PanoRtcEngineKit object.
///        @~chinese PanoRtcEngineKit 对象的配置类。 */
@JsonSerializable(explicitToJson: true)
class RtcEngineConfig {
  /// @brief @~english The application ID applied from PANO.
  ///        @~chinese 从PANO申请的应用标识。 */
  String appId;

  /// @brief @~english The PANO server address. Format: <"domain name">[:port].
  ///        @~chinese PANO服务器地址。格式：<"域名">[:端口] */
  String rtcServer;

  /// @brief @~english Whether the video codec is enabled for hardware acceleration. Default: NO.
  ///        @~chinese 视频编解码器是否启用硬件加速。默认值：否。 */
  bool videoCodecHwAcceleration;

  /// @brief @~english The audio Scenario. Default: 0(voip).
  ///        @~chinese 音频场景。默认值：0(voip) */
  int audioScenario;

  /// Constructs a [RtcEngineConfig]
  RtcEngineConfig(this.appId, this.rtcServer,
      {this.videoCodecHwAcceleration = false, this.audioScenario = 0});

  /// @nodoc
  factory RtcEngineConfig.fromJson(Map<String, dynamic> json) =>
      _$RtcEngineConfigFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcEngineConfigToJson(this);
}

/// @brief @~english The configurations class for joinning a channel.
///        @~chinese 用于加入频道的配置类。 */
@JsonSerializable(explicitToJson: true)
class RtcChannelConfig {
  /// @brief @~english Channel working mode. Default: ChannelMode.OneOnOne.
  ///        @~chinese 频道工作模式。默认值：ChannelMode.OneOnOne。 */
  ChannelMode mode;

  /// @brief @~english Channel service serviceFlags. Default: ChannelService.Media | ChannelService.Whiteboard.
  ///        @~chinese 频道服务标志。默认值：ChannelService.Media | ChannelService.Whiteboard。 */
  Set<ChannelService> serviceFlags;

  /// @brief @~english Whether to subscribe audio automatically. Default: YES.
  ///        @~chinese 是否自动订阅所有音频。默认值：是。 */
  bool subscribeAudioAll;

  /// @brief @~english The user display name. It must compliance with the following rules:
  ///                     - max length is 128 bytes.
  ///                     - UTF8 string.
  ///        @~chinese 用户显示名字。必须符合以下规则:
  ///                     - 最大长度是128字节；
  ///                     - UTF8 字符串。*/
  String userName;

  /// Constructs a [RtcChannelConfig]
  RtcChannelConfig(
      {this.mode = ChannelMode.OneOnOne,
      this.serviceFlags = const {
        ChannelService.Media,
        ChannelService.Whiteboard
      },
      this.subscribeAudioAll = true,
      this.userName});

  /// @nodoc
  factory RtcChannelConfig.fromJson(Map<String, dynamic> json) =>
      _$RtcChannelConfigFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcChannelConfigToJson(this);
}

/// @brief @~english The configurations class of the video renderer. @~chinese 视频渲染器的配置类。 */
@JsonSerializable(explicitToJson: true)
class RtcRenderConfig {
  /// @brief @~english The video profile. Default: kPanoProfileLowest.
  ///        @~chinese 视频设定档。默认值：kPanoProfileLowest。 */
  VideoProfileType profileType;

  /// @~english Enable Video Source Mirror. Default: NO @~chinese  启用视频源镜像, 默认值: NO。*/
  bool sourceMirror;

  /// @brief @~english The video scaling mode. Default: kPanoScalingFit.
  ///        @~chinese 视频缩放模式。默认值：kPanoScalingFit。 */
  VideoScalingMode scalingMode;

  /// @brief @~english Whether to enable video mirroring. Default: NO.
  ///        @~chinese 是否启用视频镜像。默认值：NO。 */
  bool mirror;

  /// Constructs a [RtcRenderConfig]
  RtcRenderConfig(
      {this.profileType = VideoProfileType.Lowest,
      this.sourceMirror = false,
      this.scalingMode = VideoScalingMode.Fit,
      this.mirror = false});

  /// @nodoc
  factory RtcRenderConfig.fromJson(Map<String, dynamic> json) =>
      _$RtcRenderConfigFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcRenderConfigToJson(this);
}

/// @brief @~english The audio format class. @~chinese 音频格式类。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioFormat {
  /// @brief @~english The audio type. Default: kPanoPCM.
  ///        @~chinese 音频类型。默认值：kPanoPCM。 */
  final AudioType type;

  /// @brief @~english The number of audio channels.
  ///        @~chinese 音频通道数。 */
  final int channels;

  /// @brief @~english The audio sample rate.
  ///        @~chinese 音频采样率。 */
  final int sampleRate;

  /// @brief @~english The bytes per audio frame.
  ///        @~chinese 音频每帧的字节数。 */
  final int bytesPerSample;

  /// Constructs a [RtcAudioFormat]
  RtcAudioFormat(
      {this.type = AudioType.PCM,
      this.channels = 0,
      this.sampleRate = 0,
      this.bytesPerSample = 0});

  /// @nodoc
  factory RtcAudioFormat.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioFormatFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioFormatToJson(this);
}

/// @brief @~english The video format class. @~chinese 视频格式类。 */
@JsonSerializable(explicitToJson: true)
class RtcVideoFormat {
  /// @brief @~english The video type. Default: kPanoI420.
  ///        @~chinese 视频类型。默认值：kPanoI420。 */
  final VideoType type;

  /// @brief @~english The video width.
  ///        @~chinese 视频宽度。 */
  final int width;

  /// @brief @~english The video height.
  ///        @~chinese 视频高度。 */
  final int height;

  /// @~english @brief The count of video block array.
  /// @details If the video type is kPanoI420, the count should be 3.
  /// @~chinese @brief 视频块数组的项目数。
  /// @details 如果视频类型为kPanoI420，则计数应为3。 */
  final int count;

  /// @~english @brief The video block offset array. Item type: UInt32.
  /// @details The offsets are for the first address of the video block.
  /// @~chinese @brief 视频块偏移量数组。项目类型：UInt32。
  /// @details 偏移量都是针对视频块首地址的。 */
  final List<int> offset;

  /// @brief @~english The video block stride array. Item type: UInt32.
  ///        @~chinese 视频块步幅数组。项目类型：UInt32。 */
  final List<int> stride;

  /// @brief @~english The video rotation degrees. Default: kPanoRotation0.
  ///        @~chinese 视频旋转角度。默认值：kPanoRotation0。 */
  final VideoRotation rotation;

  /// Constructs a [RtcVideoFormat]
  RtcVideoFormat(this.offset, this.stride,
      {this.type = VideoType.I420,
      this.width = 0,
      this.height = 0,
      this.count = 0,
      this.rotation = VideoRotation.Rotation0});

  /// @nodoc
  factory RtcVideoFormat.fromJson(Map<String, dynamic> json) =>
      _$RtcVideoFormatFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcVideoFormatToJson(this);
}

/// @brief @~english The statistics class of user audio level. @~chinese 音频接收统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioLevel {
  /// @~english @brief The user ID of received audio.
  /// @details The propertie is the user of the audio that has been subscribed.
  /// @~chinese @brief 音频接收用户标识。
  /// @details 此属性是已被订阅音频的用户。 */
  final int userId;

  /// @~english @brief The audio output strength level. Valid value range:[0, 32768].
  /// @details The propertie is the instantaneous value when callbacking statistics.
  /// @~chinese @brief 音频输出强度级别。有效值范围：[0, 32768]。
  /// @details 此属性是回调统计时的瞬时值。 */
  final int level;

  /// @~english @brief The audio active flag.
  /// @details The propertie is the instantaneous flag when callbacking statistics.
  /// @~chinese @brief 音频活跃标志。
  /// @details 此属性是回调统计时的瞬时音频活跃标志。 */
  final bool active;

  /// Constructs a [RtcAudioLevel]
  RtcAudioLevel(this.userId, {this.level = 0, this.active = false});

  /// @nodoc
  factory RtcAudioLevel.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioLevelFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioLevelToJson(this);
}

/// @brief @~english Audio profile. @~chinese 音频配置。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioProfile {
  /// @~english @brief The audio sample rate.
  /// @~chinese @brief 音频采样率。*/
  final AudioSampleRate sampleRate;

  /// @~english @brief The audio channel.
  /// @~chinese @brief 音频通道数。*/
  final AudioChannel channel;

  /// @~english @brief The audio quality Profile.
  /// @~chinese @brief 音频质量。 */
  final AudioProfileQuality profileQuality;

  /// Constructs a [RtcAudioProfile]
  RtcAudioProfile(
      {this.sampleRate = AudioSampleRate.Rate48K,
      this.channel = AudioChannel.Mono,
      this.profileQuality = AudioProfileQuality.Default});

  /// @nodoc
  factory RtcAudioProfile.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioProfileFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioProfileToJson(this);
}

/// @brief @~english The statistics class of sent audio. @~chinese 音频发送统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioSendStats {
  /// @~english @brief The audio sent bytes.
  /// @details The propertie is the total bytes after the audio is started.
  /// @~chinese @brief 音频发送字节数。
  /// @details 此属性是音频开启之后的累计字节。 */
  final int bytesSent;

  /// @~english @brief The audio sent bitrate. Unit: bps.
  /// @details The propertie is the instantaneous bitrate when callbacking statistics.
  /// @~chinese @brief 音频发送比特率。单位：比特每秒。
  /// @details 此属性是回调统计时的瞬时比特率。 */
  final int sendBitrate;

  /// @~english @brief The numnber of audio sent lost packets.
  /// @details The propertie is the total packets after the audio is started.
  /// @~chinese @brief 音频发送丢包数。
  /// @details 此属性是音频开启之后的累计包数。 */
  final int packetsLost;

  /// @~english @brief The audio sent loss ratio.
  /// @details The propertie is the instantaneous ratio when callbacking statistics.
  /// @~chinese @brief 音频发送丢包率。
  /// @details 此属性是回调统计时的瞬时比率。 */
  final double lossRatio;

  /// @~english @brief The audio round-trip time.
  /// @details The propertie is the recent RTT value when callbacking statistics.
  /// @~chinese @brief 音频往返时延。
  /// @details 此属性是回调统计时的最近往返时延值。 */
  final int rtt;

  /// @~english @brief The audio input strength level. Valid value range:[0, 32767].
  /// @details The propertie is the instantaneous value when callbacking statistics.
  /// @~chinese @brief 音频输入强度级别。有效值范围：[0, 32767]。
  /// @details 此属性是回调统计时的瞬时值。 */
  final int inputLevel;

  /// @~english @brief The local audio input active detection
  /// @details The propertie is the instantaneous value when callbacking statistics.
  /// @~chinese @brief 本地用户(麦克风采集)说话检测。
  /// @details 此属性是回调统计时的瞬时值。 */
  final bool inputActiveFlag; //  true: active, false: inactive

  /// @~english @brief The type of audio codec.
  /// @details The propertie is the dynamic value during audio sending.
  /// @~chinese @brief 音频编码器类型。
  /// @details 此属性是在音频发送期间是个动态值。 */
  final AudioCodecType codecType;

  /// Constructs a [RtcAudioSendStats]
  RtcAudioSendStats(
      this.bytesSent,
      this.sendBitrate,
      this.packetsLost,
      this.lossRatio,
      this.rtt,
      this.inputLevel,
      this.inputActiveFlag,
      this.codecType);

  /// @nodoc
  factory RtcAudioSendStats.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioSendStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioSendStatsToJson(this);
}

/// @brief @~english The statistics class of received audio. @~chinese 音频接收统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioRecvStats {
  /// @~english @brief The user ID of received audio.
  /// @details The propertie is the user of the audio that has been subscribed.
  /// @~chinese @brief 音频接收用户标识。
  /// @details 此属性是已被订阅音频的用户。 */
  final int userId;

  /// @~english @brief The audio received bytes.
  /// @details The propertie is the total bytes after the audio is subscribed.
  /// @~chinese @brief 音频接收字节数。
  /// @details 此属性是音频订阅之后的累计字节。 */
  final int bytesRecv;

  /// @~english @brief The audio received bitrate. Unit: bps.
  /// @details The propertie is the instantaneous bitrate when callbacking statistics.
  /// @~chinese @brief 音频接收比特率。单位：比特每秒。
  /// @details 此属性是回调统计时的瞬时比特率。 */
  final int recvBitrate;

  /// @~english @brief The numnber of audio received lost packets.
  /// @details The propertie is the total packets after the audio is subscribed.
  /// @~chinese @brief 音频接收丢包数。
  /// @details 此属性是音频订阅之后的累计包数。 */
  final int packetsLost;

  /// @~english @brief The audio received loss ratio.
  /// @details The propertie is the instantaneous ratio when callbacking statistics.
  /// @~chinese @brief 音频接收丢包率。
  /// @details 此属性是回调统计时的瞬时比率。 */
  final double lossRatio;

  /// @~english @brief The audio output strength level. Valid value range:[0, 32767].
  /// @details The propertie is the instantaneous value when callbacking statistics.
  /// @~chinese @brief 音频输出强度级别。有效值范围：[0, 32767]。
  /// @details 此属性是回调统计时的瞬时值。 */
  final int outputLevel;

  /// @~english @brief The type of audio codec.
  /// @details The propertie is the dynamic value during audio receiving.
  /// @~chinese @brief 音频解码器类型。
  /// @details 此属性是在音频接收期间是个动态值。 */
  final AudioCodecType codecType;

  /// Constructs a [RtcAudioRecvStats]
  RtcAudioRecvStats(this.userId, this.bytesRecv, this.recvBitrate,
      this.packetsLost, this.lossRatio, this.outputLevel, this.codecType);

  /// @nodoc
  factory RtcAudioRecvStats.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioRecvStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioRecvStatsToJson(this);
}

/// @brief @~english The statistics class of sent video. @~chinese 视频发送统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcVideoSendStats {
  /// @~english @brief The stream ID of received video.
  /// @~chinese @brief 视频流标识。*/
  final int streamId;

  /// @~english @brief The video sent bytes.
  /// @details The propertie is the total bytes after the video is started.
  /// @~chinese @brief 视频发送字节数。
  /// @details 此属性是视频开启之后的累计字节。 */
  final int bytesSent;

  /// @~english @brief The video sent bitrate. Unit: bps.
  /// @details The propertie is the instantaneous bitrate when callbacking statistics.
  /// @~chinese @brief 视频发送比特率。单位：比特每秒。
  /// @details 此属性是回调统计时的瞬时比特率。 */
  final int sendBitrate;

  /// @~english @brief The numnber of video sent lost packets.
  /// @details The propertie is the total packets after the video is started.
  /// @~chinese @brief 视频发送丢包数。
  /// @details 此属性是视频开启之后的累计包数。 */
  final int packetsLost;

  /// @~english @brief The video sent loss ratio.
  /// @details The propertie is the instantaneous ratio when callbacking statistics.
  /// @~chinese @brief 视频发送丢包率。
  /// @details 此属性是回调统计时的瞬时比率。 */
  final double lossRatio;

  /// @~english @brief The with of video sent resolution.
  /// @details The propertie is the dynamic value during video sending.
  /// @~chinese @brief 视频发送分辨率的宽度值。
  /// @details 此属性是在视频发送期间是个动态值。 */
  final int width;

  /// @~english @brief The height of video sent resolution.
  /// @details The propertie is the dynamic value during video sending.
  /// @~chinese @brief 视频发送分辨率的高度值。
  /// @details 此属性是在视频发送期间是个动态值。 */
  final int height;

  /// @~english @brief The video sent frame rate.
  /// @details The propertie is the dynamic value during video sending.
  /// @~chinese @brief 视频发送帧率。
  /// @details 此属性是在视频发送期间是个动态值。 */
  final int framerate;

  /// @~english @brief The number of received PLI packets during video sending.
  /// @details The propertie is the total packets after the video is started.
  /// @~chinese @brief 视频发送期间接收到的PLI包数。
  /// @details 此属性是视频开启之后的累计包数。 */
  final int plisReceived;

  /// @~english @brief The video round-trip time.
  /// @details The propertie is the recent RTT value when callbacking statistics.
  /// @~chinese @brief 视频往返时延。
  /// @details 此属性是回调统计时的最近往返时延值。 */
  final int rtt;

  /// @~english @brief The type of video codec.
  /// @details The propertie is the dynamic value during video sending.
  /// @~chinese @brief 视频编码器类型。
  /// @details 此属性是在视频发送期间是个动态值。 */
  final VideoCodecType codecType;

  /// Constructs a [RtcVideoSendStats]
  RtcVideoSendStats(
      this.streamId,
      this.bytesSent,
      this.sendBitrate,
      this.packetsLost,
      this.lossRatio,
      this.width,
      this.height,
      this.framerate,
      this.plisReceived,
      this.rtt,
      this.codecType);

  /// @nodoc
  factory RtcVideoSendStats.fromJson(Map<String, dynamic> json) =>
      _$RtcVideoSendStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcVideoSendStatsToJson(this);
}

/// @brief @~english The statistics class of received video. @~chinese 视频接收统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcVideoRecvStats {
  /// @~english @brief The user ID of received video.
  /// @details The propertie is the user of the video that has been subscribed.
  /// @~chinese @brief 视频接收用户标识。
  /// @details 此属性是已被订阅视频的用户。 */
  final int userId;

  /// @~english @brief The stream ID of received video.
  /// @~chinese @brief 视频流标识。*/
  final int streamId;

  /// @~english @brief The video received bytes.
  /// @details The propertie is the total bytes after the video is subscribed.
  /// @~chinese @brief 视频接收字节数。
  /// @details 此属性是视频订阅之后的累计字节。 */
  final int bytesRecv;

  /// @~english @brief The video received bitrate. Unit: bps.
  /// @details The propertie is the instantaneous bitrate when callbacking statistics.
  /// @~chinese @brief 视频接收比特率。单位：比特每秒。
  /// @details 此属性是回调统计时的瞬时比特率。 */
  final int recvBitrate;

  /// @~english @brief The numnber of video received lost packets.
  /// @details The propertie is the total packets after the video is subscribed.
  /// @~chinese @brief 视频接收丢包数。
  /// @details 此属性是视频订阅之后的累计包数。 */
  final int packetsLost;

  /// @~english @brief The video received loss ratio.
  /// @details The propertie is the instantaneous ratio when callbacking statistics.
  /// @~chinese @brief 视频接收丢包率。
  /// @details 此属性是回调统计时的瞬时比率。 */
  final double lossRatio;

  /// @~english @brief The width of video received resolution.
  /// @details The propertie is the dynamic value during video receiving.
  /// @~chinese @brief 视频接收分辨率的宽度值。
  /// @details 此属性是在视频接收期间是个动态值。 */
  final int width;

  /// @~english @brief The height of video received resolution.
  /// @details The propertie is the dynamic value during video receiving.
  /// @~chinese @brief 视频接收分辨率的高度值。
  /// @details 此属性是在视频接收期间是个动态值。 */
  final int height;

  /// @~english @brief The video received frame rate.
  /// @details The propertie is the dynamic value during video receiving.
  /// @~chinese @brief 视频接收帧率。
  /// @details 此属性是在视频接收期间是个动态值。 */
  final int framerate;

  /// @~english @brief The number of sent PLI packets during video receiving.
  /// @details The propertie is the total packets after the video is subscribed.
  /// @~chinese @brief 视频接收期间发送出的PLI包数。
  /// @details 此属性是视频订阅之后的累计包数。 */
  final int plisSent;

  /// @~english @brief The type of video codec.
  /// @details The propertie is the dynamic value during video receiving.
  /// @~chinese @brief 视频解码器类型。
  /// @details 此属性是在视频接收期间是个动态值。 */
  final VideoCodecType codecType;

  /// Constructs a [RtcVideoRecvStats]
  RtcVideoRecvStats(
      this.userId,
      this.streamId,
      this.bytesRecv,
      this.recvBitrate,
      this.packetsLost,
      this.lossRatio,
      this.width,
      this.height,
      this.framerate,
      this.plisSent,
      this.codecType);

  /// @nodoc
  factory RtcVideoRecvStats.fromJson(Map<String, dynamic> json) =>
      _$RtcVideoRecvStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcVideoRecvStatsToJson(this);
}

/// @brief @~english The statistics class of bandwidth estimation of sent video.
///        @~chinese 视频发送带宽评估统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcVideoSendBweStats {
  /// @~english @brief The evaluated bandwidth of sent video.
  /// @details The propertie is the total bandwidth of video and screen sharing.
  /// @~chinese @brief 视频发送的评估带宽。
  /// @details 此属性是视频和屏幕共享的总带宽。 */
  final int bandwidth;

  /// @~english @brief The encode bitrate of sent video. Unit: bps.
  /// @details The propertie is the total encode bitrate of video and screen sharing.
  /// @~chinese @brief 视频发送的编码比特率。单位：比特每秒。
  /// @details 此属性是视频和屏幕共享的总编码比特率。 */
  final int encodeBitrate;

  /// @~english @brief The transmit bitrate of sent video. Unit: bps.
  /// @details The propertie is the total transmit bitrate of video and screen sharing.
  /// @~chinese @brief 视频发送的传输比特率。单位：比特每秒。
  /// @details 此属性是视频和屏幕共享的总传输比特率。 */
  final int transmitBitrate;

  /// @~english @brief The retransmit bitrate of sent video. Unit: bps.
  /// @details The propertie is the total retransmit bitrate of video and screen sharing.
  /// @~chinese @brief 视频发送的重传比特率。单位：比特每秒。
  /// @details 此属性是视频和屏幕共享的总重传比特率。 */
  final int retransmitBitrate;

  /// Constructs a [RtcVideoSendBweStats]
  RtcVideoSendBweStats(this.bandwidth, this.encodeBitrate, this.transmitBitrate,
      this.retransmitBitrate);

  /// @nodoc
  factory RtcVideoSendBweStats.fromJson(Map<String, dynamic> json) =>
      _$RtcVideoSendBweStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcVideoSendBweStatsToJson(this);
}

/// @brief @~english The statistics class of bandwidth estimation of received video.
///        @~chinese 视频接收带宽评估统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcVideoRecvBweStats {
  /// @~english @brief The user ID of received video.
  /// @details The propertie is the user of the video that has been subscribed.
  /// @~chinese @brief 视频接收用户标识。
  /// @details 此属性是已被订阅视频的用户。 */
  final int userId;

  /// @~english @brief The evaluated bandwidth of received video.
  /// @details The propertie is the total bandwidth of video and screen sharing.
  /// @~chinese @brief 视频接收的评估带宽。
  /// @details 此属性是视频和屏幕共享的总带宽。 */
  final int bandwidth;

  /// Constructs a [RtcVideoRecvBweStats]
  RtcVideoRecvBweStats(this.userId, this.bandwidth);

  /// @nodoc
  factory RtcVideoRecvBweStats.fromJson(Map<String, dynamic> json) =>
      _$RtcVideoRecvBweStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcVideoRecvBweStatsToJson(this);
}

/// @brief @~english The statistics class of system information. @~chinese 系统信息统计类。 */
@JsonSerializable(explicitToJson: true)
class RtcSystemStats {
  /// @~english @brief The total cpu usage. Unit: percentage.
  /// @~chinese @brief 总CPU负载。单位：百分比。 */
  final int totalCpuUsage;

  /// @~english @brief The total physical memory. Unit: KByte.
  /// @~chinese @brief 总物理内存。单位：千字节。 */
  final int totalPhysMemory;

  /// @~english @brief The memory used by current process. Unit: KByte.
  /// @~chinese @brief 当前进程使用内存。单位：千字节。 */
  final int workingSetSize;

  /// @~english @brief The total memory usage. Unit: percentage.
  /// @~chinese @brief 总内存负载。单位：百分比。 */
  final int memoryUsage;

  /// Constructs a [RtcVideoRecvBweStats]
  RtcSystemStats(this.totalCpuUsage, this.totalPhysMemory, this.workingSetSize,
      this.memoryUsage);

  /// @nodoc
  factory RtcSystemStats.fromJson(Map<String, dynamic> json) =>
      _$RtcSystemStatsFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcSystemStatsToJson(this);
}

/// @brief @~english The device information class. @~chinese 设备信息类。 */
@JsonSerializable(explicitToJson: true)
class RtcDeviceInfo {
  /// @brief @~english The device unque ID. @~chinese 设备唯一标识。 */
  final String deviceId;

  /// @brief @~english The device display name. @~chinese 设备可显示名字。 */
  final String deviceName;

  /// Constructs a [RtcDeviceInfo]
  RtcDeviceInfo({this.deviceId, this.deviceName});

  /// @nodoc
  factory RtcDeviceInfo.fromJson(Map<String, dynamic> json) =>
      _$RtcDeviceInfoFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcDeviceInfoToJson(this);
}

/// @brief @~english The screen source information class. @~chinese 屏幕源信息类。 */
@JsonSerializable(explicitToJson: true)
class RtcScreenSourceInfo {
  /// @brief @~english The screen source unque ID. @~chinese 屏幕源唯一标识。 */
  final int sourceId;

  /// @brief @~english The screen source display name. @~chinese 屏幕源可显示名字。 */
  final String sourceName;

  /// Constructs a [RtcScreenSourceInfo]
  RtcScreenSourceInfo(this.sourceId, this.sourceName);

  /// @nodoc
  factory RtcScreenSourceInfo.fromJson(Map<String, dynamic> json) =>
      _$RtcScreenSourceInfoFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcScreenSourceInfoToJson(this);
}

/// @brief @~english The whiteboard color class. @~chinese 白板颜色类。 */
@JsonSerializable(explicitToJson: true)
class WBColor {
  /// @brief @~english The red component, valid value range:[0.0, 1.0]. Default: 0.0.
  ///        @~chinese 红色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
  double red;

  /// @brief @~english The green component, valid value range:[0.0, 1.0]. Default: 0.0.
  ///        @~chinese 绿色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
  double green;

  /// @brief @~english The blue component, valid value range:[0.0, 1.0]. Default: 0.0.
  ///        @~chinese 蓝色成分，有效值范围：[0.0, 1.0]。默认值：0.0。 */
  double blue;

  /// @brief @~english The alpha component, valid value range:[0.0, 1.0]. Default: 1.0.
  ///        @~chinese 透明度成分，有效值范围：[0.0, 1.0]。默认值：1.0。 */
  double alpha;

  /// Constructs a [WBColor]
  WBColor(
      {this.red = 0.0, this.green = 0.0, this.blue = 0.0, this.alpha = 1.0});

  /// @nodoc
  factory WBColor.fromJson(Map<String, dynamic> json) =>
      _$WBColorFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$WBColorToJson(this);
}

/// @brief @~english The whiteboard text format class. @~chinese 白板文本格式类。 */
@JsonSerializable(explicitToJson: true)
class WBTextFormat {
  /// @brief @~english The font style, PanoWBFontStyle enum type. Default: kPanoWBFontNormal.
  ///        @~chinese 字体样式，PanoWBFontStyle 枚举类型。默认值：kPanoWBFontNormal。 */
  WBFontStyle style;

  /// @brief @~english The font size, valid value range:[10, 96]. Default: 12.
  ///        @~chinese 字体大小，有效值范围：[10, 96]。默认值：12。 */
  int size;

  /// Constructs a [WBTextFormat]
  WBTextFormat({this.style = WBFontStyle.Normal, this.size = 12});

  /// @nodoc
  factory WBTextFormat.fromJson(Map<String, dynamic> json) =>
      _$WBTextFormatFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$WBTextFormatToJson(this);
}

/// @brief @~english The face beautify option class. @~chinese 美颜选项类。 */
@JsonSerializable(explicitToJson: true)
class FaceBeautifyOption {
  /// @brief @~english Whether to enable face beautify. Default: NO.
  ///        @~chinese 是否开启美颜。默认值：否。 */
  bool enable;

  /// @brief @~english The intensity of face beautify, valid value range:[0.0, 1.0]. Default: 0.5.
  ///        @~chinese 美颜强度，有效值范围：[0.0, 1.0]。默认值：0.5。 */
  double intensity;

  /// Constructs a [FaceBeautifyOption]
  FaceBeautifyOption({this.enable = false, this.intensity = 0.5});

  /// @nodoc
  factory FaceBeautifyOption.fromJson(Map<String, dynamic> json) =>
      _$FaceBeautifyOptionFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$FaceBeautifyOptionToJson(this);
}

/// @brief @~english (Deprecated) The built-in video transform option class. @~chinese （已废弃）视频内嵌变换选项类。 */
@deprecated
@JsonSerializable(explicitToJson: true)
class BuiltinTransformOption {
  /// @brief @~english Whether to enable built-in transform on video. Default: NO.
  ///        @~chinese 是否开启视频内嵌变换。默认值：否。 */
  bool enable;

  /// @brief @~english True to reset all parameters， false don't reset parameters. Default: NO.
  ///        @~chinese true 重置所有的视频形变参数， false 不重置视频形变参数。默认值：否。 */
  bool bReset;

  /// @brief @~english scaling factor in X axis (1.0: no scaling). Default: 1.0.
  ///        @~chinese X轴缩放比例 （1.0：no scaling）。默认值：1.0。 */
  double xScaling;

  /// @brief @~english scaling factor in Y axis (1.0: no scaling). Default: 1.0.
  ///        @~chinese Y轴缩放比例 （1.0：no scaling）。默认值：1.0。 */
  double yScaling;

  /// @brief @~english Delta angle of the rotation (in radians) in X axis. Default: 0.0
  ///        @~chinese X轴旋转角度的差值。默认值：0.0。 */
  double xRotation;

  /// @brief @~english Delta Angle of the rotation (in radians) in Y axis. Default: 0.0
  ///        @~chinese Y轴旋转角度的差值。默认值：0.0。 */
  double yRotation;

  /// @brief @~english Delta Angle of the rotation (in radians) in Z axis. Default: 0.0
  ///        @~chinese Z轴旋转角度的差值。默认值：0.0。 */
  double zRotation;

  /// @brief @~english Projection Depth along X axis. Default: 0.0.
  ///        @~chinese X轴的投影深度。 默认值：0.0。 */
  double xProjection;

  /// @brief @~english Projection Depth along Y axis. Default: 0.0.
  ///        @~chinese Y轴的投影深度。 默认值：0.0。 */
  double yProjection;

  /// Constructs a [BuiltinTransformOption]
  BuiltinTransformOption(
      {this.enable = false,
      this.bReset = false,
      this.xScaling = 1.0,
      this.yScaling = 1.0,
      this.xRotation = 0.0,
      this.yRotation = 0.0,
      this.zRotation = 0.0,
      this.xProjection = 0.0,
      this.yProjection = 0.0});

  /// @nodoc
  factory BuiltinTransformOption.fromJson(Map<String, dynamic> json) =>
      _$BuiltinTransformOptionFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$BuiltinTransformOptionToJson(this);
}

/// @brief @~english The quadrilateral video transform option class. @~chinese 视频四边形变换选项类。 */
@JsonSerializable(explicitToJson: true)
class QuadTransformOption {
  /// @brief @~english Whether to enable quadrilateral transform on video. Default: NO.
  ///        @~chinese 是否开启视频四边形变换。默认值：否。 */
  bool enable;

  /// @brief @~english True to reset all quadrilateral transform parameters， false don't reset quadrilateral transform parameters. Default: NO.
  ///        @~chinese true 重置所有的视频四边形形变参数， false 不重置视频四边形形变参数。默认值：否。 */
  bool bReset;

  /// @brief @~english Vertex index of a quadrilateral. TopLeft: 0, TopRight: 1, BottomLeft: 2, BottomRight: 3
  ///        @~chinese 四边形顶点索引。 左上角：0，右上角：1，左下角：2，右下角：3*/
  QuadIndex index;

  /// @brief @~english Delta of x axis, The origin (0,0) of the video is the top left, the whole size of video is 1x1, can be negative (top or left is out of view), and can be greater than 1 (bottom or right is out of view).
  ///        @~chinese X坐标轴的差值，视频的左上角为坐标系的原点（0，0），视频完整大小为1x1，可以为负值（左侧超出屏幕），可以为大于1的值（右侧超出屏幕）*/
  double xDeltaAxis;

  /// @brief @~english Delta of y axis, The origin (0,0) of the video is the top left, the whole size of video is 1x1, can be negative (top or left is out of view), and can be greater than 1 (bottom or right is out of view).
  ///        @~chinese Y坐标轴的差值，视频的左上角为坐标系的原点（0，0），视频完整大小为1x1，可以为负值（左或上超出屏幕），可以为大于1的值（右或下超出屏幕）*/
  double yDeltaAxis;

  /// @brief @~english Set Mirror mode for video transform (exchange left and right). Note: Set mirror mode to true when using front facing camera, false when using back facing camera. Reset doesn't reset this, but keeps last value.
  ///        @~chinese 设置视频变换为镜像模式(左右交换）。注意：使用前置摄像头时需要设置为true，使用后置摄像头时需要设置为false。重置参数时，并不会改变mirror的值，仍将保持上次的设置。*/
  bool bMirror;

  /// Constructs a [QuadTransformOption]
  QuadTransformOption(
      {this.enable = false,
      this.bReset = false,
      this.index = QuadIndex.TopLeft,
      this.xDeltaAxis = 0.0,
      this.yDeltaAxis = 0.0,
      this.bMirror = false});

  /// @nodoc
  factory QuadTransformOption.fromJson(Map<String, dynamic> json) =>
      _$QuadTransformOptionFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$QuadTransformOptionToJson(this);
}

/// @brief @~english Feedback info class, user can send feedback to PANO.
///        @~chinese 用户反馈信息类，用于发送用户反馈。*/
@JsonSerializable(explicitToJson: true)
class FeedbackInfo {
  /// @brief @~english Feedback type. Default: kPanoFeedbackGeneral.
  ///        @~chinese 反馈类型。默认值：kPanoFeedbackGeneral。 */
  FeedbackType type;

  /// @brief @~english Product name, max length 128 bytes.
  ///        @~chinese 产品名，最多128字节。 */
  String productName;

  /// @brief @~english Detail description of problem, max length 1024 bytes.
  ///        @~chinese 问题详细描述，最多1024字节。 */
  String detailDescription;

  /// @brief @~english User contact, max length 128 bytes.
  ///        @~chinese 联系信息，最多128字节。 */
  String contact;

  /// @brief @~english User extra info, max length 256 bytes.
  ///        @~chinese 附加信息，最多256字节。 */
  String extraInfo;

  /// @brief @~english Whether to upload logs. Default: NO.
  ///        @~chinese 是否上传日志。默认值：否。 */
  bool uploadLogs;

  /// Constructs a [FeedbackInfo]
  FeedbackInfo(this.type, this.productName, this.detailDescription,
      {this.contact, this.extraInfo, this.uploadLogs = false});

  /// @nodoc
  factory FeedbackInfo.fromJson(Map<String, dynamic> json) =>
      _$FeedbackInfoFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$FeedbackInfoToJson(this);
}

/// @brief @~english The configurations class of audio mixing. @~chinese 音频混音配置类。 */
@JsonSerializable(explicitToJson: true)
class RtcAudioMixingConfig {
  /// @brief @~english Enable publish. Default: YES
  ///        @~chinese 是否发送。默认值：是。 */
  bool enablePublish;

  /// @~english publish volume. 0~200. Default: 100.
  /// @note There could be cracking sounds when the volume is larger than 100.
  /// @~chinese 发送音量。0～200。默认值：100。
  /// @note 音量超过100后可能产生破音。
  int publishVolume;

  /// @brief @~english Enable loopback. Default: YES
  ///        @~chinese 是否发送。默认值：是。 */
  bool enableLoopback;

  /// @~english loopback volume. 0~200. Default: 100.
  /// @note There could be cracking sounds when the volume is larger than 100.
  /// @~chinese 回放音量。0～200。默认值：100。
  /// @note 音量超过100后可能产生破音。
  int loopbackVolume;

  /// @brief @~english Times to play. 0 means loop forever. Default: 1.
  ///        @~chinese 播放次数。0指无限循环。默认值：1。 */
  int cycle;

  /// @brief @~english YES: Replace microphone data. NO: Mix with microphone data. Default: NO.
  ///        @~chinese 是: 替换麦克风数据。否: 与麦克风数据混音。默认值：否。 */
  bool replaceMicrophone;

  /// Constructs a [RtcAudioMixingConfig]
  RtcAudioMixingConfig(
      {this.enablePublish = true,
      this.publishVolume = 100,
      this.enableLoopback = true,
      this.loopbackVolume = 100,
      this.cycle = 1,
      this.replaceMicrophone = false});

  /// @nodoc
  factory RtcAudioMixingConfig.fromJson(Map<String, dynamic> json) =>
      _$RtcAudioMixingConfigFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcAudioMixingConfigToJson(this);
}

/// @brief @~english The option class of video snapshot. @~chinese 视频快照选项类。 */
@JsonSerializable(explicitToJson: true)
class RtcSnapshotVideoOption {
  /// @brief @~english The format of snapshot. Default: kPanoImageFileJPEG.
  ///        @~chinese 快照格式。默认值：kPanoImageFileJPEG。 */
  ImageFileFormat format;

  /// @brief @~english Whether to mirror. Default: NO.
  ///        @~chinese 是否镜像。默认值：否。 */
  bool mirror;

  /// Constructs a [RtcSnapshotVideoOption]
  RtcSnapshotVideoOption(
      {this.format = ImageFileFormat.JPEG, this.mirror = false});

  /// @nodoc
  factory RtcSnapshotVideoOption.fromJson(Map<String, dynamic> json) =>
      _$RtcSnapshotVideoOptionFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcSnapshotVideoOptionToJson(this);
}

/// @brief @~english Network quality report. @~chinese 网络质量报告。 */
@JsonSerializable(explicitToJson: true)
class RtcNetworkQuality {
  /// @brief @~english Quality rating. Default: kPanoQualityUnavailable.
  ///        @~chinese 网络质量评分。默认值：kPanoQualityUnavailable。 */
  final QualityRating rating;

  /// @brief @~english Uplink loss rate. Default: 0.
  ///        @~chinese 上行丢包率。默认值：0。 */
  final double txLoss;

  /// @brief @~english Downlink loss rate. Default: 0.
  ///        @~chinese 下行丢包率。默认值：0。 */
  final double rxLoss;

  /// @brief @~english Round-Trip Time in millisecond. Default: 0.
  ///        @~chinese RTT延迟, 单位：毫秒。默认值：0。 */
  final int rtt;

  /// Constructs a [RtcNetworkQuality]
  RtcNetworkQuality(
      {this.rating = QualityRating.Unavailable,
      this.txLoss = 0,
      this.rxLoss = 0,
      this.rtt = 0});

  /// @nodoc
  factory RtcNetworkQuality.fromJson(Map<String, dynamic> json) =>
      _$RtcNetworkQualityFromJson(json);

  /// @nodoc
  Map<String, dynamic> toJson() => _$RtcNetworkQualityToJson(this);
}
