import 'package:flutter/services.dart';

import '../pano_rtc.dart';
import 'enum_converter.dart';

class RtcVideoStreamManager with RtcVideoStreamInterface {
  final MethodChannel _methodChannel = MethodChannel('pano_rtc/api_video_streamMgr');
  final EventChannel _eventChannel = EventChannel('pano_rtc/events_video_streamMgr');
  VideoStreamEventHandler _handler;

  RtcVideoStreamManager() {
    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  Future<T> _invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  void setEventHandler(VideoStreamEventHandler handler) {
    _handler = handler;
  }

  @override
  Future<int> createVideoStream(String deviceId) {
    return _invokeMethod('createVideoStream', {'deviceId': deviceId});
  }

  @override
  Future<ResultCode> destroyVideoStream(int streamId) {
    return _invokeMethod('destroyVideoStream', {'streamId': streamId});
  }

  @override
  Future<String> getCaptureDevice(int streamId) {
    return _invokeMethod(
        'getCaptureDevice', {'streamId': streamId});
  }

  @override
  Future<ResultCode> muteVideo(int streamId) {
    return _invokeMethod('muteVideo', {'streamId': streamId});
  }

  @override
  Future<ResultCode> setCaptureDevice(int streamId, String deviceId) {
    return _invokeMethod(
        'setCaptureDevice', {'streamId': streamId, 'deviceId': deviceId});
  }

  @override
  Future<ResultCode> snapshotVideo(int userId, int streamId, String outputDir,
      RtcSnapshotVideoOption option) {
    return _invokeMethod('snapshotVideo', {
      'userId': userId,
      'streamId': streamId,
      'outputDir': outputDir,
      'option': option.toJson()
    });
  }

  @override
  Future<ResultCode> startVideo(
      int streamId, RtcSurfaceViewModel viewModel, RtcRenderConfig config) {
    return viewModel.invokeMethod(
        'startVideoWithStreamId', {'streamId': streamId, 'config': config.toJson()});
  }

  @override
  Future<ResultCode> stopVideo(int streamId) {
    return _invokeMethod('stopVideo', {'streamId': streamId});
  }

  @override
  Future<ResultCode> subscribeVideoWithStreamId(int userId, int streamId,
      RtcSurfaceViewModel viewModel, RtcRenderConfig config) {
    return viewModel.invokeMethod('subscribeVideoWithStreamId',
        {'userId': userId, 'streamId': streamId, 'config': config.toJson()});
  }

  @override
  Future<ResultCode> unmuteVideo(int streamId) {
    return _invokeMethod('unmuteVideo', {'streamId': streamId});
  }

  @override
  Future<ResultCode> unsubscribeVideo(int userId, int streamId) {
    return _invokeMethod(
        'unsubscribeVideo', {'userId': userId, 'streamId': streamId});
  }
}

mixin RtcVideoStreamInterface {
  ///@~english
  ///@brief Create a new video stream.
  ///@param deviceId  The device to be set to new stream.
  ///@return
  ///   - >= 0: video stream ID
  ///   - others: Failure
  ///@note The default video stream is always available after channel joined
  ///@~chinese
  ///@brief 创建一个新的视频流。
  ///@param deviceId  设备 ID, 此设备会设置给新视频流。
  ///@return
  ///   - >= 0: 视频流 ID
  ///  - 其他: 失败
  ///@note 默认视频流无需创建，在频道加入成功后即有效。
  Future<int> createVideoStream(String deviceId);

  ///@~english
  ///@brief Destroy a video stream.
  ///@param streamId  The stream ID to be destroyed.
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note The default video stream could not be destroyed
  ///@~chinese
  ///@brief 销毁一个视频流。
  ///@param streamId  视频流 ID。
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 默认视频流不可销毁。
  Future<ResultCode> destroyVideoStream(int streamId);

  ///@~english
  ///@brief Set capture device for video stream.
  ///@param streamId  Stream ID.
  ///@param deviceId  Device ID.
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@~chinese
  ///@brief 设置或更新视频流的采集设备。
  ///@param streamId  视频流 ID。
  ///@param deviceId  采集设备 ID。
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  Future<ResultCode> setCaptureDevice(int streamId, String deviceId);

  ///@~english
  ///@brief Get capture device of the stream.
  ///@param streamId  Stream ID.
  ///@return
  ///  - empty: Failure, the streamId is not found
  ///  - others: The capture device ID
  ///@~chinese
  ///@brief 获取视频流的采集设备。
  ///@param streamId  视频流 ID。
  ///@return
  ///  - 空字符串: 失败，streamId未找到或其他错误
  ///  - 非空字符串: 成功，返回采集设备ID
  Future<String> getCaptureDevice(int streamId);

  ///@~english
  ///@brief start video stream with render window
  ///@param streamId Stream ID
  ///@param window Platform specific window handle, it should be HWND on windows platform.
  ///@param config Video configure
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note Must be called from main thread on platform macOS iOS and Android.
  ///@~chinese
  ///@brief 开启视频流，并且设置渲染窗口
  ///@param streamId 视频流 ID
  ///@param window 平台相关的窗口句柄， 在windows平台， 该参数必须是HWND
  ///@param config 视频配置参数
  ///@return
  ///  - OK: 调用成功
  ///  - others： 调用失败
  ///@note 在 macOS iOS and Android 平台下必须从主线程调用。
  Future<ResultCode> startVideo(
      int streamId, RtcSurfaceViewModel viewModel, RtcRenderConfig config);

  ///@~english
  ///@brief Stop video stream
  ///@param streamId Stream ID
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@~chinese
  ///@brief 关闭视频流
  ///@param streamId 视频流 ID
  ///@return
  ///  - OK: 调用成功
  ///  - others： 调用失败
  Future<ResultCode> stopVideo(int streamId);

  ///@~english
  ///@brief Pause video stream.
  ///@param streamId Stream ID
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note Please start the video stream before pausing, otherwise it will not work.
  ///@~chinese
  ///@brief 暂停视频流。
  ///@param streamId 视频流 ID
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 暂停视频流前请先开启视频流，否则操作将无效。
  Future<ResultCode> muteVideo(int streamId);

  ///@~english
  ///@brief Resume video stream.
  ///@param streamId Stream ID
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note Please start the video stream before resuming, otherwise it will not work.
  ///@~chinese
  ///@brief 恢复视频。
  ///@param streamId 视频流 ID
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 恢复视频流前请先开启视频流，否则操作将无效。
  Future<ResultCode> unmuteVideo(int streamId);

  ///@~english
  ///@brief Subscribe to a user's video stream with render window.
  ///@param userId  The user ID defined by customer.
  ///@param streamId Stream ID.
  ///@param window  Platform specified window object, it should HWND on windows platform.
  ///@param config  Video configure.
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note Before subscribing to a user’s video stream, please make sure the user has started the video stream.
  ///      Must be called from main thread on platform macOS iOS and Android.
  ///@~chinese
  ///@brief 订阅用户的视频流， 并设置渲染窗口。
  ///@param userId  客户定义的用户标识。
  ///@param streamId 视频流 ID。
  ///@param window  平台相关的窗口对象， 在windows平台， 该参数必须是HWND对象。
  ///@param config  视频参数。
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 订阅用户的视频流前，请确保用户已开启视频流。
  ///      在 macOS iOS and Android 平台下必须从主线程调用。
  Future<ResultCode> subscribeVideoWithStreamId(int userId, int streamId,
      RtcSurfaceViewModel viewModel, RtcRenderConfig config);

  ///@~english
  ///@brief Unsubscribe to a user's video stream.
  ///@param userId  The user ID defined by customer.
  ///@param streamId  Stream ID.
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@note When a user stops the video or leaves the channel, the user's video will be automatically unsubscribed.
  ///@~chinese
  ///@brief 取消订阅用户的视频。
  ///@param userId  客户定义的用户标识。
  ///@param streamId 视频流 ID。
  ///@return
  ///  - OK: 成功
  ///  - 其他: 失败
  ///@note 当用户停止视频流或者离开频道的时候，用户的视频流将会被自动取消订阅。
  Future<ResultCode> unsubscribeVideo(int userId, int streamId);

  ///@~english
  ///@brief Capture specific user's video stream content
  ///@param userId    the id of target user
  ///@param streamId  the video stream ID
  ///@param outputDir output directory
  ///@param option    snapshot option
  ///@return
  ///  - OK: Success
  ///  - others: Failure
  ///@~chinese
  ///@brief 捕获指定用户的视频流画面
  ///@param userId    目标用户的 ID
  ///@param streamId  视频流 ID
  ///@param outputDir 输出路径
  ///@param option    快照选项
  ///@return
  /// - OK: 成功
  /// - 其他: 失败
  Future<ResultCode> snapshotVideo(int userId, int streamId, String outputDir,
      RtcSnapshotVideoOption option);
}
