import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/services.dart';
import 'package:pano_rtc/pano_rtc.dart';
import 'package:pano_rtc/src/enum_converter.dart';

import 'rtc_enums.dart';
import 'rtc_objects.dart';
import 'rtc_view.dart';

class RtcWhiteboard with RtcWhiteboardInterface {
  MethodChannel _methodChannel;
  EventChannel _eventChannel;
  WhiteboardEventHandler _handler;

  Future<T> _invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  RtcWhiteboard(String channelId) {
    _methodChannel = MethodChannel("pano_rtc/api_whiteboard_$channelId");
    _eventChannel = EventChannel("pano_rtc/events_whiteboard_$channelId");

    _eventChannel.receiveBroadcastStream().listen((event) {
      final eventMap = Map<dynamic, dynamic>.from(event);
      final methodName = eventMap['methodName'] as String;
      final data = List<dynamic>.from(eventMap['data']);
      _handler?.process(methodName, data);
    });
  }

  void setEventHandler(WhiteboardEventHandler handler) {
    _handler = handler;
  }

  @override
  Future<ResultCode> open(RtcWhiteboardSurfaceViewModel viewModel) {
    return viewModel.invokeMethod('open');
  }

  @override
  Future<ResultCode> close() {
    return _invokeMethod('close');
  }

  @override
  Future<ResultCode> stop() {
    return _invokeMethod('stop');
  }

  @override
  Future<ResultCode> setRoleType(WBRoleType type) {
    return _invokeMethod(
        'setRoleType', {'type': WBRoleTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> setToolType(WBToolType type) {
    return _invokeMethod(
        'setToolType', {'type': WBToolTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> setLineWidth(int width) {
    return _invokeMethod('setLineWidth', {'width': width});
  }

  @override
  Future<ResultCode> setFillType(WBFillType type) {
    return _invokeMethod(
        'setFillType', {'type': WBFillTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> setFillColor(WBColor color) {
    return _invokeMethod('setFillColor', {'color': color.toJson()});
  }

  @override
  Future<ResultCode> setForegroundColor(WBColor color) {
    return _invokeMethod('setForegroundColor', {'color': color.toJson()});
  }

  @override
  Future<ResultCode> setBackgroundColor(WBColor color) {
    return _invokeMethod('setBackgroundColor', {'color': color.toJson()});
  }

  @override
  Future<ResultCode> setFontStyle(WBFontStyle style) {
    return _invokeMethod(
        'setFontStyle', {'style': WBFontStyleConverter(style).value()});
  }

  @override
  Future<ResultCode> setFontSize(int size) {
    return _invokeMethod('setFontSize', {'size': size});
  }

  @override
  Future<ResultCode> setBackgroundImageScalingMode(WBImageScalingMode mode) {
    return _invokeMethod('setBackgroundImageScalingMode',
        {'mode': WBImageScalingModeConverter(mode).value()});
  }

  @override
  Future<ResultCode> setBackgroundImage(String imageUrl) {
    return _invokeMethod('setBackgroundImage', {'imageUrl': imageUrl});
  }

  @override
  Future<ResultCode> setBackgroundImageWithPage(String imageUrl, int pageNo) {
    return _invokeMethod(
        'setBackgroundImageWithPage', {'imageUrl': imageUrl, 'pageNo': pageNo});
  }

  @override
  Future<int> getCurrentPageNumber() {
    return _invokeMethod('getCurrentPageNumber');
  }

  @override
  Future<int> getTotalNumberOfPages() {
    return _invokeMethod('getTotalNumberOfPages');
  }

  @override
  Future<ResultCode> addPage(bool autoSwitch) {
    return _invokeMethod('addPage', {'autoSwitch': autoSwitch});
  }

  @override
  Future<ResultCode> insertPage(int pageNo, bool autoSwitch) {
    return _invokeMethod(
        'insertPage', {'pageNo': pageNo, 'autoSwitch': autoSwitch});
  }

  @override
  Future<ResultCode> removePage(int pageNo, {bool switchNext = false}) {
    return _invokeMethod(
        'removePage', {'pageNo': pageNo, 'switchNext': switchNext});
  }

  @override
  Future<ResultCode> gotoPage(int pageNo) {
    return _invokeMethod('gotoPage', {'pageNo': pageNo});
  }

  @override
  Future<ResultCode> nextPage() {
    return _invokeMethod('nextPage');
  }

  @override
  Future<ResultCode> prevPage() {
    return _invokeMethod('prevPage');
  }

  @override
  Future<String> addBackgroundImages(List<String> urls) {
    return _invokeMethod('addBackgroundImages', {'urls': urls});
  }

  @override
  Future<String> createDocWithImages(List<String> urls) {
    return _invokeMethod('createDocWithImages', {'urls': urls});
  }

  @override
  Future<String> createDocWithFilePath(String filePath) {
    return _invokeMethod('createDocWithFilePath', {'filePath': filePath});
  }

  @override
  Future<ResultCode> deleteDoc(String fileId) {
    return _invokeMethod('deleteDoc', {'fileId': fileId});
  }

  @override
  Future<ResultCode> switchDoc(String fileId) {
    return _invokeMethod('switchDoc', {'fileId': fileId});
  }

  @override
  Future<ResultCode> saveDocToImages(String fileId, String outputDir) {
    return _invokeMethod(
        'saveDocToImages', {'fileId': fileId, 'outputDir': outputDir});
  }

  @override
  Future<List<String>> enumerateFiles() {
    return _invokeMethod('enumerateFiles');
  }

  @override
  Future<String> getCurrentFileId() {
    return _invokeMethod('getCurrentFileId');
  }

  @override
  Future<ResultCode> clearContents(bool curPage, WBClearType type) {
    return _invokeMethod('clearContents',
        {'curPage': curPage, 'type': WBClearTypeConverter(type).value()});
  }

  @override
  Future<ResultCode> clearUserContents(
      int userId, bool curPage, WBClearType type) {
    return _invokeMethod('clearUserContents', {
      'userId': userId,
      'curPage': curPage,
      'type': WBClearTypeConverter(type).value()
    });
  }

  @override
  Future<ResultCode> undo() {
    return _invokeMethod('undo');
  }

  @override
  Future<ResultCode> redo() {
    return _invokeMethod('redo');
  }

  @override
  Future<double> getCurrentScaleFactor() {
    return _invokeMethod('getCurrentScaleFactor');
  }

  @override
  Future<ResultCode> setCurrentScaleFactor(double scale) {
    return _invokeMethod('setCurrentScaleFactor', {'scale': scale});
  }

  @override
  Future<ResultCode> snapshot(WBSnapshotMode mode, String outputDir) {
    return _invokeMethod('snapshot', {
      'mode': WBSnapshotModeConverter(mode).value(),
      'outputDir': outputDir
    });
  }

  @override
  Future<ResultCode> sendMessage(Uint8List message, int userId) {
    return _invokeMethod('sendMessage', {'message': message, 'userId': userId});
  }

  @override
  Future<ResultCode> broadcastMessage(Uint8List message) {
    return _invokeMethod('broadcastMessage', {'message': message});
  }

  @override
  Future<ResultCode> setOption(option, WBOptionType type) {
    var params = Map<String, dynamic>();
    params["type"] = WBOptionTypeConverter(type).value();
    var isValid = true;
    switch (type) {
      case WBOptionType.FileCachePath:
        if (option is String) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      case WBOptionType.EnableUIResponse:
        if (option is bool) {
          params["option"] = option;
        } else {
          isValid = false;
        }
        break;
      default:
        isValid = false;
    }

    if (!isValid) return Future.value(ResultCode.InvalidArgs);

    return _invokeMethod('setOption', params);
  }
}

/// @~english
/// @brief The PanoRtcWhiteboard interface class provides all whitboard methods invoked by the application.
/// @details The PanoRtcWhiteboard object can be obtained through the PanoRtcEngineKit instance.
/// @~chinese
/// @brief PanoRtcWhiteboard 接口类对应用程序提供了所有的白板相关方法。
/// @details PanoRtcWhiteboard 对象可以通过 PanoRtcEngineKit 实例获取。
mixin RtcWhiteboardInterface {
  /// @~english
  /// @brief Open the whiteboard.
  /// @param view  The whiteboard display view provided by customer.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 打开白板。
  /// @param view  客户提供的白板显示视图。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> open(RtcWhiteboardSurfaceViewModel viewModel);

  /// @~english
  /// @brief Close the whiteboard.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 关闭白板。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> close();

  /// @~english
  /// @brief Stop the whiteboard.
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @note default whiteboard can't be stopped
  /// @~chinese
  /// @brief 停止白板
  /// @return
  ///   - OK： 成功
  ///   - Others: 失败
  /// @note 默认白板不能被停止
  Future<ResultCode> stop();

  /// @~english
  /// @brief Set whiteboard role type.
  /// @param type  The whiteboard role type, PanoWBToolType enum type.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板角色类型。
  /// @param type  白板角色，PanoWBRoleType 枚举类型。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setRoleType(WBRoleType type);

  /// @~english
  /// @brief Set the whiteboard tool.
  /// @param type  PanoWBToolType enum type.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板工具。
  /// @param view  PanoWBToolType 枚举类型。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setToolType(WBToolType type);

  /// @~english
  /// @brief Set the whiteboard line width.
  /// @param width   Valid value range: [1, 20].
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板线条宽度。
  /// @param width   有效值范围：[1, 20]。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setLineWidth(int width);

  /// @~english
  /// @brief Set the whiteboard fill type.
  /// @param type  PanoWBFillType enum type.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板填充类型。
  /// @param type  PanoWBFillType 枚举类型。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setFillType(WBFillType type);

  /// @~english
  /// @brief Set the whiteboard fill color.
  /// @param color   PanoWBColor object. Valid value range: [0, 1].
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @note Only works when the fill type is set to kPanoWBFillColor.
  /// @~chinese
  /// @brief 设置白板填充颜色。
  /// @param color   PanoWBColor 对象。有效值范围：[0, 1]。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  /// @note 仅当设置填充类型为kPanoWBFillColor起效。
  Future<ResultCode> setFillColor(WBColor color);

  /// @~english
  /// @brief Set the whiteboard foreground color.
  /// @param color   PanoWBColor object. Valid value range: [0, 1].
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板前景色。
  /// @param color   PanoWBColor 对象。有效值范围：[0, 1]。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setForegroundColor(WBColor color);

  /// @~english
  /// @brief Set the whiteboard background color.
  /// @param color   PanoWBColor object. Valid value range: [0, 1].
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板背景色。
  /// @param color   PanoWBColor 对象。有效值范围：[0, 1]。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setBackgroundColor(WBColor color);

  /// @~english
  /// @brief Set the whiteboard font style.
  /// @param style   PanoWBFontStyle enum type.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板字体样式。
  /// @param color   PanoWBFontStyle 枚举类型。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setFontStyle(WBFontStyle style);

  /// @~english
  /// @brief Set the font size.
  /// @param size  Valid value range: [10, 96].
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置字体大小。
  /// @param size  有效值范围：[10, 96]。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setFontSize(int size);

  /// @~english
  /// @brief Set background image scaling mode.
  /// @param mode  background image scaling mode.
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @~chinese
  /// @brief 设置白板背景图缩放模式。
  /// @param mode  背景图缩放模式。
  /// @return
  ///   - OK： 成功
  ///   - Others: 失败
  Future<ResultCode> setBackgroundImageScalingMode(WBImageScalingMode mode);

  /// @~english
  /// @brief Set the background image of current whiteboard page.
  /// @param imageUrl  Image URL, a local iamge path or a network image link.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板当前页背景图片。
  /// @param imageUrl  图片 URL，可为本地路径或者网络链接。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setBackgroundImage(String imageUrl);

  /// @~english
  /// @brief Set background image of specified whiteboard page.
  /// @param imageUrl  Image URL, a local iamge path or a network image link.
  /// @param pageNo    The page number.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板指定页背景图片
  /// @param imageUrl  背景图 URL，可为本地路径或者网络链接。
  /// @param pageNo    白板页码。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setBackgroundImageWithPage(String imageUrl, int pageNo);

  /// @~english
  /// @brief Get current page number.
  /// @return
  ///    - PanoWBPageNumber(-1): No page
  ///    - Others: Page number
  /// @~chinese
  /// @brief 获取当前白板页码。
  /// @return
  ///   - PanoWBPageNumber(-1)：白板未打开
  ///   - Others：白板页码
  Future<int> getCurrentPageNumber();

  /// @~english
  /// @brief Get total numbar of pages.
  /// @return The number of pages
  /// @~chinese
  /// @brief 获取总白板页码数。
  /// @return 页码数
  Future<int> getTotalNumberOfPages();

  /// @~english
  /// @brief Add new page to the end.
  /// @param autoSwitch    Auto switch to the new page.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 添加一个新页面到最后。
  /// @param autoSwitch    自动切换到新添加的页面。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> addPage(bool autoSwitch);

  /// @~english
  /// @brief Add new page after the page number.
  /// @param pageNo        The page number.
  /// @param autoSwitch    Auto switch to the new page.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 添加一个新页面到指定页码之后。
  /// @param pageNo        指定页码。
  /// @param autoSwitch    自动切换到新添加的页面。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> insertPage(int pageNo, bool autoSwitch);

  /// @~english
  /// @brief goto the page pageNo
  /// @param pageNo the page
  /// @param switchNext switch to next page of removed page.
  /// @note The default behavior is switch to previous page of removed page after remove page.
  /// Set switchNext true to make it switch to next page of removed page.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 删除页 pageNo
  /// @param pageNo 被删除的页码
  /// @param switchNext 切换到删除页的下一页.
  /// @note 删除页的默认行为是切换到删除页的前一页。设置switchNext为true改为切换到删除页的下一页。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> removePage(int pageNo, {bool switchNext = false});

  /// @~english
  /// @brief Goto the page with the pange number.
  /// @param pageNo The page number.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 切换到指定页。
  /// @param pageNo 指定页码。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> gotoPage(int pageNo);

  /// @~english
  /// @brief Switch to next page.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 切换到下一页。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> nextPage();

  /// @~english
  /// @brief Switch to previous page.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 切换到前一页。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> prevPage();

  /// @~english
  /// @brief Add some background images to current whiteboard file
  /// @param urls Background image url array (remote url only)
  /// @return
  ///    - Current whiteborad file ID, if fail return empty string
  /// @note PanoWhiteboard has created doc with whiteboard file ID "default" when created
  /// @~chinese
  /// @brief 添加指定数量的背景图到当前白板文件
  /// @param urls 背景图url数组（仅支持远程URL）
  /// @return
  ///    - 当前白板文件ID，如果失败返回空串
  /// @note PanoWhiteboard 创建时会生成白板文件ID为"default"的白板文件
  Future<String> addBackgroundImages(List<String> urls);

  /// @~english
  /// @brief Create new whiteboard file with some background images
  /// @param urls Background image url array (remote url only)
  /// @return
  ///    - Whiteborad file ID created, if fail return empty string
  /// @note PanoWhiteboard has created doc with whiteboard file ID "default" when created
  /// @~chinese
  /// @brief 导入指定数量的背景图并创建新的白板文件
  /// @param urls 背景图url数组（仅支持远程URL）
  /// @return
  ///    - 新创建的白板文件ID，如果失败返回空串
  /// @note PanoWhiteboard 创建时会生成白板文件ID为"default"的白板文件
  Future<String> createDocWithImages(List<String> urls);

  /// @~english
  /// @brief Upload local file for transcode and create new whiteboard file
  /// @param filePath Local file path
  /// @return
  ///    - Whiteborad file ID created, if fail return empty string
  /// @note PanoWhiteboard has created doc with whiteboard file ID "default" when created
  /// @~chinese
  /// @brief 上传本地需转码的文件并创建新的白板文件
  /// @param filePath 本地文件路径
  /// @return
  ///    - 新创建的白板文件ID，如果失败返回空串
  /// @note PanoWhiteboard 创建时会生成白板文件ID为"default"的白板文件
  Future<String> createDocWithFilePath(String filePath);

  /// @~english
  /// @brief Delete whiteboard file
  /// @param fileId Whiteboard file ID
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @note "default" whiteboard file could not be deleted
  /// @~chinese
  /// @brief 删除白板文件
  /// @param fileId 白板文件ID
  /// @return
  ///    - OK： 成功
  ///    - Others: 失败
  /// @note "default" 白板文件不能被删除
  Future<ResultCode> deleteDoc(String fileId);

  /// @~english
  /// @brief Switch whiteboard file
  /// @param fileId Whiteboard file ID
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @~chinese
  /// @brief 切换白板文件
  /// @param fileId 白板文件ID
  /// @return
  ///    - OK： 成功
  ///    - Others: 失败
  Future<ResultCode> switchDoc(String fileId);

  /// @~english
  /// @brief Save whiteboard file. Save each page to one image.
  /// Image name format is like whiteboard_[fileId]_[page number].png, eg. whiteboard_default_1.png
  /// @param fileId Whiteboard file ID
  /// @param outputDir Output directory
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @~chinese
  /// @brief 保存白板文件。每个白板页存为一张图
  /// 图像名称格式为whiteboard_[fileId]_[page number].png, 例如：whiteboard_default_1.png
  /// @param fileId 白板文件ID
  /// @param outputDir 输出路径
  /// @return
  ///    - OK： 成功
  ///    - Others: 失败
  Future<ResultCode> saveDocToImages(String fileId, String outputDir);

  /// @~english
  /// @brief Enumerate whiteboard files
  /// @return fileId array
  /// @~chinese
  /// @brief 枚举白板文件
  /// @return fileId 数组
  Future<List<String>> enumerateFiles();

  /// @~english
  /// @brief get current whiteboard file ID
  /// @return
  ///    - current whiteborad file ID, if fail return null
  /// @~chinese
  /// @brief 获取当前白板文件ID
  /// @return
  ///    - 当前白板文件ID, 失败则返回null
  Future<String> getCurrentFileId();

  /// @~english
  /// @brief clear whiteboard content
  /// @param curPage true: clear current page only; false: clear all pages
  /// @param type WBClearType enum type
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @note The operation requires admin role.
  /// @~chinese
  /// @brief 清除白板内容
  /// @param curPage true: 只清除当前页内容；false: 清除所有页内容
  /// @param type WBClearType 枚举类型
  /// @return
  ///   - OK： 成功
  ///   - Others: 失败
  /// @note 此操作需要管理员角色。
  Future<ResultCode> clearContents(bool curPage, WBClearType type);

  /// @~english
  /// @brief clear whiteboard content by specific user ID
  /// @param userId user ID
  /// @param curPage true: clear current page only; false: clear all pages
  /// @param type WBClearType enum type
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @note If the user isn't current user, the operation requires admin role.
  /// @~chinese
  /// @brief 清除指定用户绘制的白板内容
  /// @param userId 用户ID
  /// @param curPage true: 只清除当前页内容；false: 清除所有页内容
  /// @param type WBClearType 枚举类型
  /// @return
  ///   - OK： 成功
  ///   - Others: 失败
  /// @note 如果指定用户不是当前用户，此操作需要管理员角色。
  Future<ResultCode> clearUserContents(
      int userId, bool curPage, WBClearType type);

  /// @~english
  /// @brief Undo the last operation of the whiteboard.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 撤销白板上一次操作。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> undo();

  /// @~english
  /// @brief Redo the last undone operation of the whiteboard.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 重做白板的上一次被撤销操作。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> redo();

  /// @~english
  /// @brief Get current whiteboard scale factor.
  /// @return scale factor.
  /// @~chinese
  /// @brief 获取当前白板视图的缩放比例。
  /// @return 缩放比例值。
  Future<double> getCurrentScaleFactor();

  /// @~english
  /// @brief set current whiteboard scale factor
  /// @param scale scale factor. Valid value range [0.1, 5.0]
  /// @return
  ///    - OK: Success
  ///    - Others: Fail
  /// @~chinese
  /// @brief 设置当前白板视图的缩放比例
  /// @param scale 缩放比例值。有效值范围 [0.1, 5.0]
  /// @return
  ///   - OK： 成功
  ///   - Others: 失败
  Future<ResultCode> setCurrentScaleFactor(double scale);

  /// @~english
  /// @brief Save whtieboard contents to image.
  /// @param mode snapshot mode
  /// @param outputDir Output directory.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @note snapshot result and image filename is returned in delegate method onSnapshotComplete: name:
  /// @~chinese
  /// @brief 保存白板内容到图像。
  /// @param mode 快照模式
  /// @param outputDir 输出路径。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  /// @note 快照结果和图像文件名通过回调函数onSnapshotComplete: name:返回
  Future<ResultCode> snapshot(WBSnapshotMode mode, String outputDir);

  /// @~english
  /// @brief Send message to the user specified by userId.
  /// @param message  The message data.
  /// @param userId   The user who will receive the message.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 发送消息给某个指定用户。
  /// @param message  要发送的消息。
  /// @param userId   接收消息的用户。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> sendMessage(Uint8List message, int userId);

  /// @~english
  /// @brief Broadcast message to all users.
  /// @param message  The message data.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 广播消息给所有用户。
  /// @param message  要广播的消息。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> broadcastMessage(Uint8List message);

  /// @~english
  /// @brief Set whiteboard option object.
  /// @param option  The Option object.
  /// @param type    PanoWBOptionType enum type.
  /// @return
  ///   - kPanoResultOK: Success
  ///   - others: Failure
  /// @~chinese
  /// @brief 设置白板选项对象。
  /// @param option  选项对象。
  /// @param type    PanoWBOptionType 枚举类型。
  /// @return
  ///   - kPanoResultOK: 成功
  ///   - 其他: 失败
  Future<ResultCode> setOption(dynamic option, WBOptionType type);
}
